from typing import List

from openapi_client.models.orm_kernel_resource_spec import OrmKernelResourceSpec
from vessl import vessl_api
from vessl.organization import _get_organization_name
from vessl.util.exception import InvalidKernelResourceSpecError


def read_kernel_resource_spec(
    kernel_resource_spec_name: str, **kwargs
) -> OrmKernelResourceSpec:
    """Read kernel resource spec

    Keyword args:
        organization_name (str): override default organization
    """
    kernel_resource_specs = list_kernel_resource_specs(**kwargs)
    kernel_resource_specs = {x.name: x for x in kernel_resource_specs}

    if kernel_resource_spec_name not in kernel_resource_specs:
        raise InvalidKernelResourceSpecError(
            f"Kernel resource spec not found: {kernel_resource_spec_name}"
        )
    return kernel_resource_specs[kernel_resource_spec_name]


def list_kernel_resource_specs(**kwargs) -> List[OrmKernelResourceSpec]:
    """List kernel resource specs

    Keyword args:
        organization_name (str): override default organization
    """
    return vessl_api.kernel_resource_spec_list_api(
        organization_name=_get_organization_name(**kwargs),
    ).results


def _configure_custom_kernel_resource_spec(
    processor_type: str = None,
    cpu_limit: float = None,
    memory_limit: float = None,
    gpu_type: str = None,
    gpu_limit: int = None,
) -> OrmKernelResourceSpec:
    return OrmKernelResourceSpec(
        processor_type=processor_type,
        cpu_type="Any",
        cpu_limit=cpu_limit,
        memory_limit=f"{memory_limit}Gi",
        gpu_type=gpu_type,
        gpu_limit=gpu_limit,
    )



import os
from typing import Optional

from openapi_client.models import ResponseProjectInfo
from vessl import vessl_api
from vessl.util.constant import PROJECT_TYPE_CLI_DRIVEN
from vessl.util.exception import GitError
from vessl.util.git_repo import GitRepository
from vessl.util.uploader import Uploader


def get_git_branch(project: ResponseProjectInfo, **kwargs) -> Optional[str]:
    if project.type == PROJECT_TYPE_CLI_DRIVEN:
        return None

    if "git_branch" in kwargs:
        return kwargs["git_branch"]
    if vessl_api.default_git_repo is not None:
        return vessl_api.default_git_repo.branch
    raise GitError("No git branch selected.")


def get_git_ref(project: ResponseProjectInfo, **kwargs) -> Optional[str]:
    if project.type == PROJECT_TYPE_CLI_DRIVEN:
        return None

    if vessl_api.default_git_repo is None:
        raise GitError("No git commit selected.")

    if "git_ref" in kwargs:
        git_ref = kwargs["git_ref"]
        if vessl_api.default_git_repo.check_revision_in_remote(git_ref):
            return git_ref
        raise GitError(f"Git commit does not exist in a remote repository: {git_ref}")

    return vessl_api.default_git_repo.commit_ref


def get_git_diff_path(project: ResponseProjectInfo, **kwargs) -> Optional[str]:
    use_git_diff = kwargs.get("use_git_diff", False)
    use_git_diff_untracked = kwargs.get("use_git_diff_untracked", False)

    if not use_git_diff or project.type == PROJECT_TYPE_CLI_DRIVEN:
        return None

    if vessl_api.default_git_repo is None:
        raise GitError("No git repo")

    local_file = GitRepository.get_current_diff_file(
        revision_or_branch=get_git_ref(project), with_untracked=use_git_diff_untracked
    )

    remote_file = Uploader.upload(
        local_path=local_file.name,
        volume_id=project.volume_id,
        remote_path=os.path.basename(local_file.name),
    )

    local_file.close()
    return remote_file.path



import os

import click
import inquirer
from sshpubkeys import SSHKey, InvalidKeyError

from vessl.cli._base import VesslGroup, vessl_option
from vessl.cli._util import (
    print_table,
    prompt_choices,
    truncate_datetime,
)
from vessl.ssh_key import create_ssh_key, delete_ssh_key, list_ssh_keys
from vessl.util.constant import SSH_PUBLIC_KEY_PATH
from vessl.util.datetime import parse_time_to_ago


def ssh_key_id_callback(ctx: click.Context, param: click.Parameter, value: id) -> id:
    ssh_keys = list_ssh_keys()
    return prompt_choices(
        "SSH key", [(f"{x.name} / {x.fingerprint} (created {parse_time_to_ago(x.created_dt)})", x.id) for x in ssh_keys]
    )


def ssh_public_key_value_callback(ctx: click.Context, param: click.Option, key_path: str) -> str:
    if not key_path:
        key_path = inquirer.prompt([inquirer.Text(
            "question",
            message="SSH public key path",
            default=SSH_PUBLIC_KEY_PATH,
        )], raise_keyboard_interrupt=True).get("question")

    try:
        with open(key_path, "r") as f:
            ssh_public_key_value = f.read()
    except FileNotFoundError:
        raise click.BadParameter("Key file not found")

    ssh = SSHKey(ssh_public_key_value, strict=True)
    try:
        ssh.parse()
    except InvalidKeyError as e:
        raise click.BadParameter(f"Invalid key: {e}")
    except NotImplementedError as e:
        raise click.BadParameter(f"Invalid key type: {e}")

    ctx.obj["ssh_key_name"] = ssh.comment
    ctx.obj["ssh_key_filename"] = os.path.basename(key_path)
    return ssh_public_key_value


def ssh_key_name_callback(ctx: click.Context, param: click.Option, name: str) -> str:
    if name:
        return name

    return inquirer.prompt([inquirer.Text(
        "question",
        message="SSH public key name",
        default=ctx.obj.get("ssh_key_name"),
    )], raise_keyboard_interrupt=True).get("question")


@click.command(name="ssh-key", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command()
def list():
    ssh_keys = list_ssh_keys()
    print_table(
        ssh_keys,
        ["Name", "Fingerprint", "Created"],
        lambda x: [x.name, x.fingerprint, truncate_datetime(x.created_dt)],
    )


@cli.vessl_command()
@vessl_option(
    "-p",
    "--path",
    "ssh_public_key_value",
    type=click.Path(exists=True),
    help="Path to SSH public key.",
    callback=ssh_public_key_value_callback,
)
@vessl_option(
    "--name",
    type=click.STRING,
    callback=ssh_key_name_callback,
)
@click.pass_context
def add(ctx: click.Context, name: str, ssh_public_key_value: str):
    ssh_key = create_ssh_key(key_name=name, key_path=ctx.obj["ssh_key_filename"], ssh_public_key_value=ssh_public_key_value)
    print(f"SSH Key '{ssh_key.name}' created.")


@cli.vessl_command()
@vessl_option(
    "--id",
    type=click.INT,
    hidden=True,
    callback=ssh_key_id_callback,
)
def delete(id: int):
    delete_ssh_key(key_id=id)
    print("Successfully deleted.")



from typing import List

import keras
import numpy as np

from vessl.experiment import list_experiments
from vessl.integration.common import get_current_experiment, log
from vessl.model import create_model
from vessl.util import logger
from vessl.util.constant import (
    VESSL_PLOTS_FILETYPE_IMAGE,
    VESSL_PLOTS_FILETYPE_IMAGES,
)
from vessl.util.exception import InvalidExperimentError, InvalidParamsError
from vessl.util.image import Image


class ExperimentCallback(keras.callbacks.Callback):
    def __init__(
        self,
        data_type=None,
        validation_data=None,
        num_images=None,
        labels=None,
        start_epoch=0,
        save_image=False,
    ):
        super().__init__()
        self._data_type = data_type
        self._num_images = num_images or 1
        self._labels = labels
        self._start_epoch = start_epoch
        self._save_image = save_image

        self.validation_data = None
        if validation_data is not None:
            self.validation_data = validation_data

    def _results_to_predicts(self, results):
        predicts = []
        if results[0].shape[-1] == 1:
            if len(self._labels) == 2:
                predicts = [
                    self._lables[1] if result[0] > 0.5 else self._labels[0]
                    for result in results
                ]
            else:
                if not self._labels:
                    logger.warn("Cannot find labels for prediction")
                predicts = [result[0] for result in results]
        else:
            argmax_results = np.argmax(np.stack(results), axis=1)
            if not self._labels:
                logger.warn("Cannot find labels for prediction")
                predicts = argmax_results
            else:
                for argmax_result in argmax_results:
                    try:
                        predicts.append(self._labels[argmax_result])
                    except IndexError:
                        predicts.append(argmax_result)
        return predicts

    def _inference(self):
        x_val, y_val = self.validation_data

        if self._num_images > len(x_val):
            self._num_images = len(x_val)

        random_indices = np.random.choice(len(x_val), self._num_images, replace=False)
        x_val_random = [x_val[i] for i in random_indices]
        y_val_random = [y_val[i] for i in random_indices]

        results = self.model.predict(np.stack(x_val_random), batch_size=1)
        predicts = self._results_to_predicts(results)

        captions = []
        for predict, truth in zip(predicts, y_val_random):
            captions.append(f"Pred: {predict} Truth: {truth}")

        return [Image(x, caption=caption) for x, caption in zip(x_val_random, captions)]

    def on_epoch_end(self, epoch, logs=None):
        log(step=epoch + self._start_epoch + 1, row=logs)

        if self._save_image and self._data_type in (
            VESSL_PLOTS_FILETYPE_IMAGE,
            VESSL_PLOTS_FILETYPE_IMAGES,
        ):
            if self.validation_data is None:
                logger.warn("Cannot find validation_data")

            log({"validation_image": self._inference()})


MODEL_UPLOAD_POLICY_ALWAYS: str = "always"
MODEL_UPLOAD_POLICY_BEST_ONLY: str = "best_only"
MODEL_UPLOAD_OBJECTIVE_MAXIMIZE: str = "maximize"
MODEL_UPLOAD_OBJECTIVE_MINIMIZE: str = "minimize"


class ModelCallback(keras.callbacks.Callback):
    def __init__(
        self,
        upload_policy: str = MODEL_UPLOAD_POLICY_ALWAYS,
        objective: str = None,
        target_metric: str = None,
        description: str = None,
        tags: List[str] = [],
    ):
        super().__init__()
        if upload_policy not in [
            MODEL_UPLOAD_POLICY_ALWAYS,
            MODEL_UPLOAD_POLICY_BEST_ONLY,
        ]:
            raise InvalidParamsError(
                "upload_policy: should be one of: "
                f"{MODEL_UPLOAD_POLICY_ALWAYS}, {MODEL_UPLOAD_POLICY_BEST_ONLY}"
            )
        if upload_policy == MODEL_UPLOAD_POLICY_BEST_ONLY:
            if objective not in [
                MODEL_UPLOAD_OBJECTIVE_MAXIMIZE,
                MODEL_UPLOAD_OBJECTIVE_MINIMIZE,
            ]:
                raise InvalidParamsError(
                    "objective: should be one of: "
                    f"{MODEL_UPLOAD_OBJECTIVE_MAXIMIZE}, {MODEL_UPLOAD_OBJECTIVE_MINIMIZE}"
                )

        self._upload_policy = upload_policy
        self._objective = objective
        self._target_metric = target_metric
        self._description = description
        self._tags = tags

    def _should_upload_model(self, logs=None) -> bool:
        if self._upload_policy == MODEL_UPLOAD_POLICY_ALWAYS:
            return True

        target_metric_name = self._target_metric
        if target_metric_name not in logs:
            logger.warn(f"metric {target_metric_name} not in train logs: {logs}")
            return False

        # Get previous best model and compare with trained model
        # TODO: compare with previous models rather than experiments
        prev_experiments = list_experiments(
            order_field=f"metrics_summary.latest.{target_metric_name}.value",
            order_direction="desc"
            if self._objective == MODEL_UPLOAD_OBJECTIVE_MAXIMIZE
            else "asc",
            limit=1,
        )
        if len(prev_experiments) == 0:
            # No best model found: upload this model
            return True

        prev_metric = prev_experiments[0].metrics_summary["latest"]
        if target_metric_name not in prev_metric:
            raise InvalidExperimentError(
                f"Expected metric value not found from experiment: {target_metric_name}"
            )
        prev_metric_value = prev_metric[target_metric_name]["value"]
        if (
            self._objective == MODEL_UPLOAD_OBJECTIVE_MAXIMIZE
            and logs[target_metric_name] >= prev_metric_value
        ):
            return True
        if (
            self._objective == MODEL_UPLOAD_OBJECTIVE_MINIMIZE
            and logs[target_metric_name] <= prev_metric_value
        ):
            return True
        return False

    def on_train_end(self, logs=None):
        experiment = get_current_experiment()
        if experiment is None:
            return

        if self._should_upload_model(logs):
            create_model(
                experiment=experiment.id,
                description=self._description,
                tags=self._tags,
            )



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponseKernelClusterInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'created_dt': 'datetime',
        'current_system_metrics': 'list[InfluxdbCurrentSystemMetric]',
        'docs_link': 'str',
        'id': 'int',
        'is_savvihub_managed': 'bool',
        'kubernetes_namespace': 'str',
        'kubernetes_service_type': 'str',
        'message': 'str',
        'name': 'str',
        'provider': 'str',
        'region': 'str',
        'status': 'str',
        'updated_dt': 'datetime'
    }

    attribute_map = {
        'created_dt': 'created_dt',
        'current_system_metrics': 'current_system_metrics',
        'docs_link': 'docs_link',
        'id': 'id',
        'is_savvihub_managed': 'is_savvihub_managed',
        'kubernetes_namespace': 'kubernetes_namespace',
        'kubernetes_service_type': 'kubernetes_service_type',
        'message': 'message',
        'name': 'name',
        'provider': 'provider',
        'region': 'region',
        'status': 'status',
        'updated_dt': 'updated_dt'
    }

    def __init__(self, created_dt=None, current_system_metrics=None, docs_link=None, id=None, is_savvihub_managed=None, kubernetes_namespace=None, kubernetes_service_type=None, message=None, name=None, provider=None, region=None, status=None, updated_dt=None, local_vars_configuration=None):  # noqa: E501
        """ResponseKernelClusterInfo - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._created_dt = None
        self._current_system_metrics = None
        self._docs_link = None
        self._id = None
        self._is_savvihub_managed = None
        self._kubernetes_namespace = None
        self._kubernetes_service_type = None
        self._message = None
        self._name = None
        self._provider = None
        self._region = None
        self._status = None
        self._updated_dt = None
        self.discriminator = None

        self.created_dt = created_dt
        if current_system_metrics is not None:
            self.current_system_metrics = current_system_metrics
        self.docs_link = docs_link
        self.id = id
        self.is_savvihub_managed = is_savvihub_managed
        self.kubernetes_namespace = kubernetes_namespace
        self.kubernetes_service_type = kubernetes_service_type
        self.message = message
        self.name = name
        self.provider = provider
        self.region = region
        self.status = status
        self.updated_dt = updated_dt

    @property
    def created_dt(self):
        """Gets the created_dt of this ResponseKernelClusterInfo.  # noqa: E501


        :return: The created_dt of this ResponseKernelClusterInfo.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ResponseKernelClusterInfo.


        :param created_dt: The created_dt of this ResponseKernelClusterInfo.  # noqa: E501
        :type created_dt: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_dt is None:  # noqa: E501
            raise ValueError("Invalid value for `created_dt`, must not be `None`")  # noqa: E501

        self._created_dt = created_dt

    @property
    def current_system_metrics(self):
        """Gets the current_system_metrics of this ResponseKernelClusterInfo.  # noqa: E501


        :return: The current_system_metrics of this ResponseKernelClusterInfo.  # noqa: E501
        :rtype: list[InfluxdbCurrentSystemMetric]
        """
        return self._current_system_metrics

    @current_system_metrics.setter
    def current_system_metrics(self, current_system_metrics):
        """Sets the current_system_metrics of this ResponseKernelClusterInfo.


        :param current_system_metrics: The current_system_metrics of this ResponseKernelClusterInfo.  # noqa: E501
        :type current_system_metrics: list[InfluxdbCurrentSystemMetric]
        """

        self._current_system_metrics = current_system_metrics

    @property
    def docs_link(self):
        """Gets the docs_link of this ResponseKernelClusterInfo.  # noqa: E501


        :return: The docs_link of this ResponseKernelClusterInfo.  # noqa: E501
        :rtype: str
        """
        return self._docs_link

    @docs_link.setter
    def docs_link(self, docs_link):
        """Sets the docs_link of this ResponseKernelClusterInfo.


        :param docs_link: The docs_link of this ResponseKernelClusterInfo.  # noqa: E501
        :type docs_link: str
        """

        self._docs_link = docs_link

    @property
    def id(self):
        """Gets the id of this ResponseKernelClusterInfo.  # noqa: E501


        :return: The id of this ResponseKernelClusterInfo.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ResponseKernelClusterInfo.


        :param id: The id of this ResponseKernelClusterInfo.  # noqa: E501
        :type id: int
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def is_savvihub_managed(self):
        """Gets the is_savvihub_managed of this ResponseKernelClusterInfo.  # noqa: E501


        :return: The is_savvihub_managed of this ResponseKernelClusterInfo.  # noqa: E501
        :rtype: bool
        """
        return self._is_savvihub_managed

    @is_savvihub_managed.setter
    def is_savvihub_managed(self, is_savvihub_managed):
        """Sets the is_savvihub_managed of this ResponseKernelClusterInfo.


        :param is_savvihub_managed: The is_savvihub_managed of this ResponseKernelClusterInfo.  # noqa: E501
        :type is_savvihub_managed: bool
        """
        if self.local_vars_configuration.client_side_validation and is_savvihub_managed is None:  # noqa: E501
            raise ValueError("Invalid value for `is_savvihub_managed`, must not be `None`")  # noqa: E501

        self._is_savvihub_managed = is_savvihub_managed

    @property
    def kubernetes_namespace(self):
        """Gets the kubernetes_namespace of this ResponseKernelClusterInfo.  # noqa: E501


        :return: The kubernetes_namespace of this ResponseKernelClusterInfo.  # noqa: E501
        :rtype: str
        """
        return self._kubernetes_namespace

    @kubernetes_namespace.setter
    def kubernetes_namespace(self, kubernetes_namespace):
        """Sets the kubernetes_namespace of this ResponseKernelClusterInfo.


        :param kubernetes_namespace: The kubernetes_namespace of this ResponseKernelClusterInfo.  # noqa: E501
        :type kubernetes_namespace: str
        """

        self._kubernetes_namespace = kubernetes_namespace

    @property
    def kubernetes_service_type(self):
        """Gets the kubernetes_service_type of this ResponseKernelClusterInfo.  # noqa: E501


        :return: The kubernetes_service_type of this ResponseKernelClusterInfo.  # noqa: E501
        :rtype: str
        """
        return self._kubernetes_service_type

    @kubernetes_service_type.setter
    def kubernetes_service_type(self, kubernetes_service_type):
        """Sets the kubernetes_service_type of this ResponseKernelClusterInfo.


        :param kubernetes_service_type: The kubernetes_service_type of this ResponseKernelClusterInfo.  # noqa: E501
        :type kubernetes_service_type: str
        """
        if self.local_vars_configuration.client_side_validation and kubernetes_service_type is None:  # noqa: E501
            raise ValueError("Invalid value for `kubernetes_service_type`, must not be `None`")  # noqa: E501

        self._kubernetes_service_type = kubernetes_service_type

    @property
    def message(self):
        """Gets the message of this ResponseKernelClusterInfo.  # noqa: E501


        :return: The message of this ResponseKernelClusterInfo.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this ResponseKernelClusterInfo.


        :param message: The message of this ResponseKernelClusterInfo.  # noqa: E501
        :type message: str
        """

        self._message = message

    @property
    def name(self):
        """Gets the name of this ResponseKernelClusterInfo.  # noqa: E501


        :return: The name of this ResponseKernelClusterInfo.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ResponseKernelClusterInfo.


        :param name: The name of this ResponseKernelClusterInfo.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def provider(self):
        """Gets the provider of this ResponseKernelClusterInfo.  # noqa: E501


        :return: The provider of this ResponseKernelClusterInfo.  # noqa: E501
        :rtype: str
        """
        return self._provider

    @provider.setter
    def provider(self, provider):
        """Sets the provider of this ResponseKernelClusterInfo.


        :param provider: The provider of this ResponseKernelClusterInfo.  # noqa: E501
        :type provider: str
        """
        if self.local_vars_configuration.client_side_validation and provider is None:  # noqa: E501
            raise ValueError("Invalid value for `provider`, must not be `None`")  # noqa: E501

        self._provider = provider

    @property
    def region(self):
        """Gets the region of this ResponseKernelClusterInfo.  # noqa: E501


        :return: The region of this ResponseKernelClusterInfo.  # noqa: E501
        :rtype: str
        """
        return self._region

    @region.setter
    def region(self, region):
        """Sets the region of this ResponseKernelClusterInfo.


        :param region: The region of this ResponseKernelClusterInfo.  # noqa: E501
        :type region: str
        """

        self._region = region

    @property
    def status(self):
        """Gets the status of this ResponseKernelClusterInfo.  # noqa: E501


        :return: The status of this ResponseKernelClusterInfo.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this ResponseKernelClusterInfo.


        :param status: The status of this ResponseKernelClusterInfo.  # noqa: E501
        :type status: str
        """
        if self.local_vars_configuration.client_side_validation and status is None:  # noqa: E501
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501
        allowed_values = ["connected", "not-connected", "permission-required", "agent-not-available", "prometheus-not-available", "modifying"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and status not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `status` ({0}), must be one of {1}"  # noqa: E501
                .format(status, allowed_values)
            )

        self._status = status

    @property
    def updated_dt(self):
        """Gets the updated_dt of this ResponseKernelClusterInfo.  # noqa: E501


        :return: The updated_dt of this ResponseKernelClusterInfo.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this ResponseKernelClusterInfo.


        :param updated_dt: The updated_dt of this ResponseKernelClusterInfo.  # noqa: E501
        :type updated_dt: datetime
        """
        if self.local_vars_configuration.client_side_validation and updated_dt is None:  # noqa: E501
            raise ValueError("Invalid value for `updated_dt`, must not be `None`")  # noqa: E501

        self._updated_dt = updated_dt

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseKernelClusterInfo):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseKernelClusterInfo):
            return True

        return self.to_dict() != other.to_dict()



from test.conftest import USE_MOCK

import pytest

import vessl


@pytest.mark.skipif(USE_MOCK, reason="Does not run if mocking is used.")
class TestKernelCluster:
    def test_read_cluster(self):
        vessl.read_cluster("aws-uw2-dev")  # TODO: change when CI server is relocated

    def test_list_clusters(self):
        vessl.list_clusters()

    def test_delete_cluster(self):
        # Do not test delete cluster because only one cluster exists.
        pass

    def test_rename_cluster(self):
        # Only applies to custom clusters
        pass

    def test_list_cluster_nodes(self):
        # Only applies to custom clusters
        pass



from typing import List

from openapi_client.models import (
    OrganizationCreateAPIPayload,
    ResponseOrganization,
    ResponseOrganizationInfo,
)
from vessl import vessl_api
from vessl.util.exception import InvalidOrganizationError


def read_organization(organization_name: str) -> ResponseOrganizationInfo:
    return vessl_api.organization_read_api(organization_name=organization_name)


def list_organizations() -> List[ResponseOrganization]:
    return vessl_api.organization_list_api().organizations


def create_organization(
    organization_name: str, region: str
) -> ResponseOrganizationInfo:
    return vessl_api.organization_create_api(
        organization_create_api_payload=OrganizationCreateAPIPayload(
            name=organization_name, region=region
        )
    )


def _get_organization_name(**kwargs) -> str:
    organization_name = kwargs.get("organization_name")
    if organization_name is not None:
        return organization_name
    if vessl_api.organization is not None:
        return vessl_api.organization.name
    raise InvalidOrganizationError("No organization selected.")



from datetime import datetime
from typing import Any, Callable, Dict, List, Union

import click
import inquirer
from terminaltables import AsciiTable

from openapi_client.models import ResponseFileMetadata
from vessl.util.constant import WEB_HOST

TAB_SIZE = 2
UNDEFINED = click.style("undefined", fg="red")


class Endpoint:
    dataset = WEB_HOST + "/{}/datasets/{}"  # Organization name, dataset name
    experiment = (
        WEB_HOST + "/{}/{}/experiments/{}"
    )  # Organization name, project name, experiment number
    experiment_logs = (
        WEB_HOST + "/{}/{}/experiments/{}/logs"
    )  # Organization name, project name, experiment number
    organization = WEB_HOST + "/{}"  # Organization name
    project = WEB_HOST + "/{}/{}"  # Organization name, project name
    sweep = WEB_HOST + "/{}/{}/sweeps/{}"  # Organization name, project name, sweep name
    sweep_logs = (
        WEB_HOST + "/{}/{}/sweeps/{}/logs"
    )  # Organization name, project name, sweep name
    workspace = WEB_HOST + "/{}/workspaces/{}"  # Organization name, workspace id


def format_key(key: str) -> str:
    return click.style(key, fg="green")


def print_table(
    objects: List[Any], keys: List[str], data_func: Callable, **kwargs
) -> None:
    table_data = [[format_key(x) for x in keys]]
    table_data.extend(data_func(x) for x in objects)

    table = AsciiTable(table_data)
    table.inner_column_border = kwargs.get("inner_column_border", False)
    table.inner_heading_row_border = kwargs.get("inner_heading_row_border", False)
    table.inner_footing_row_border = kwargs.get("inner_footing_row_border", False)
    table.outer_border = kwargs.get("outer_border", False)

    print(table.table)


def format_data(data: Any, depth: int) -> List[str]:
    indent = " " * (TAB_SIZE * depth + 1)

    if not isinstance(data, dict):
        # `data` is primitive type
        return [f"{indent}{data}"]

    lines = []
    for key, value in data.items():
        if isinstance(value, dict):
            lines.append(f"{indent}{format_key(key)}")
            lines.extend(format_data(value, depth + 1))
        elif isinstance(value, list):
            lines.append(f"{indent}{format_key(key)}")
            for x in value:
                # TODO: find a more elegant way. Perhaps yaml library?
                additional_lines = format_data(x, depth + 1)
                additional_lines[0] = additional_lines[0].lstrip()
                additional_lines[0] = f"{indent}- {additional_lines[0]}"
                lines.extend(additional_lines)
        else:
            lines.append(f"{indent}{format_key(key)} {value}")
    return lines


def print_data(data: Dict[str, Any]) -> None:
    lines = format_data(data, 0)
    print("\n".join(lines))


def print_logs(logs: List[Any]):
    timezone = datetime.now().astimezone().tzinfo
    lines = [
        f" [{datetime.fromtimestamp(x.timestamp, tz=timezone).strftime('%H:%M:%S.%f')}] {x.message}"
        for x in logs
    ]
    print("\n".join(lines))


def truncate_datetime(value: datetime) -> Union[str, datetime]:
    if not value:
        return UNDEFINED
    return value.replace(microsecond=0)


def format_size(value: int, suffix="B") -> str:
    if value == 0:
        return "0 B"

    for unit in ["", "K", "M", "G", "T", "P", "E", "Z"]:
        if abs(value) < 1024.0:
            return f"{value:.1f} {unit}{suffix}"
        value /= 1024.0
    return f"{value:.1f} Yi{suffix}"


def format_bool(value: bool, true_value: str = "Y", false_value: str = "N") -> str:
    return true_value if value else false_value


def format_string(value: str, null_value: str = "None", empty_value: str = "-") -> str:
    if value is None:
        return null_value
    if not value:
        return empty_value
    return value


def print_volume_files(
    files: List[ResponseFileMetadata],
    keys: List[str] = None,
    data_func: Callable = None,
    **kwargs,
) -> None:
    """Print volume files

    Separately defined as a util function because it is used often across many files.
    TODO: Support recursive. Add Sort? Parse paths?
    """

    if keys is None:
        keys = ["Path", "Dir", "Size"]

    if data_func is None:
        data_func = lambda x: [
            x.path,
            format_bool(x.is_dir),
            format_size(x.size) if not x.is_dir else "-",
        ]

    print_table(files, keys, data_func)


def prompt_choices(text: str, choices: List[Any], default: Any = None) -> Any:
    """Prompt choices

    Args:
        choices (list): A list of choices to display, or a list of 2-tuples where
                        the first element is the choice to display and the second
                        element is return value.
    """
    key = "question"
    inquiry = inquirer.List(key, message=text, default=default, choices=choices)
    return inquirer.prompt([inquiry], raise_keyboard_interrupt=True).get(key)


def generic_prompter(text: str, type: click.ParamType = click.STRING, default=None) -> Callable:
    def prompter(ctx: click.Context, param: click.Parameter, value: str):
        return click.prompt(text, type=type, default=default)

    return prompter


def choices_prompter(text: str, choices: List[Any], default: Any = None) -> Callable:
    def prompter(ctx: click.Context, param: click.Parameter, value: str):
        return prompt_choices(text, choices, default)

    return prompter



import os
import urllib.parse
from pathlib import Path
from typing import List
from zipfile import ZipFile

import click
import inquirer
import paramiko

from openapi_client.models import ResponseWorkspaceDetail, ResponseWorkspaceList
from vessl import vessl_api
from vessl.organization import _get_organization_name
from vessl.util.config import VesslConfigLoader
from vessl.util.constant import (
    SSH_CONFIG_FORMAT,
    SSH_CONFIG_PATH,
    WORKSPACE_TEMP_ZIP_PATH,
    WORKSPACE_ZIP_PATH,
)
from vessl.util.datetime import parse_time_to_ago
from vessl.util.exception import InvalidWorkspaceError
from vessl.util.zipper import Zipper
from vessl.volume import copy_volume_file


def read_workspace(workspace_id: int, **kwargs) -> ResponseWorkspaceDetail:
    """Read workspace

    Keyword args:
        organization_name (str): override default organization
    """
    return vessl_api.workspace_read_api(
        workspace_id=workspace_id, organization_name=_get_organization_name(**kwargs)
    )


def list_workspaces(
    cluster_id: int = None, statuses: List[str] = None, mine: bool = True, **kwargs
) -> List[ResponseWorkspaceList]:
    """List workspaces

    Keyword args:
        organization_name (str): override default organization
    """
    return vessl_api.workspace_list_api(
        organization_name=_get_organization_name(**kwargs),
        cluster=cluster_id,
        mine=mine,
        statuses=statuses,
    ).results


def backup_workspace() -> None:
    """Backup the home directory of the workspace

    Should only be called inside a workspace.
    """

    workspace_id = VesslConfigLoader().workspace
    if workspace_id is None:
        raise InvalidWorkspaceError("Can only be called within a workspace.")

    workspace = read_workspace(workspace_id)

    zipper = Zipper(WORKSPACE_TEMP_ZIP_PATH, "w")
    zipper.zipdir(str(Path.home()), zipper)
    zipper.close()

    copy_volume_file(
        source_volume_id=None,
        source_path=zipper.filename,
        dest_volume_id=workspace.backup_volume_id,
        dest_path=os.path.basename(zipper.filename),
    )

    vessl_api.cli_workspace_backup_api(workspace_id=workspace_id)


def restore_workspace() -> None:
    """Restore the home directory from the previous backup

    Should only be called inside a workspace.
    """

    workspace_id = VesslConfigLoader().workspace
    if workspace_id is None:
        raise InvalidWorkspaceError("Can only be called within a workspace.")

    workspace_list = list_workspaces(mine=True)
    backup_workspaces = [w for w in workspace_list if w.last_backup_dt is not None]
    if len(backup_workspaces) == 0:
        raise click.ClickException('Available workspace backup not found.')

    workspace = inquirer.prompt([inquirer.List(
        'question',
        message='Select workspace',
        choices=[(f'{w.name} (backup created {parse_time_to_ago(w.last_backup_dt)})', w) for w in backup_workspaces],
    )], raise_keyboard_interrupt=True).get('question')

    copy_volume_file(
        source_volume_id=workspace.backup_volume_id,
        source_path=WORKSPACE_ZIP_PATH,
        dest_volume_id=None,
        dest_path=WORKSPACE_TEMP_ZIP_PATH,
    )

    z = ZipFile(WORKSPACE_TEMP_ZIP_PATH)
    z.extractall(str(Path.home()))


def connect_workspace_ssh(private_key_path: str, **kwargs) -> None:
    """Connect to a running workspace via SSH

    Keyword args:
        organization_name (str): override default organization
    """
    running_workspaces = list_workspaces(statuses=["running"], mine=True)
    if len(running_workspaces) == 0:
        raise click.ClickException('There is no running workspace.')

    if len(running_workspaces) == 1:
        workspace = running_workspaces[0]
    else:
        workspace = inquirer.prompt([inquirer.List(
            'question',
            message='Select workspace',
            choices=[(f'{w.name} (created {parse_time_to_ago(w.created_dt)})', w) for w in running_workspaces],
        )], raise_keyboard_interrupt=True).get('question')

    ssh_endpoint = urllib.parse.urlparse(workspace.endpoints.ssh.endpoint)
    ssh_private_key_option = f' -i {private_key_path}' if private_key_path else ''
    os.system(f"ssh -p {ssh_endpoint.port}{ssh_private_key_option} vessl@{ssh_endpoint.hostname}")


def update_vscode_remote_ssh(private_key_path: str) -> None:
    """Update .ssh/config file for VSCode Remote-SSH plugin"""
    running_workspaces = list_workspaces(statuses=["running"], mine=True)
    if len(running_workspaces) == 0:
        raise click.ClickException('There is no running workspace.')

    ssh_config = paramiko.SSHConfig()
    ssh_config.parse(open(SSH_CONFIG_PATH, "r"))
    host_set = ssh_config.get_hostnames()

    for workspace in running_workspaces:
        host = f"{workspace.name}-{int(workspace.created_dt.timestamp())}"
        if host in host_set:
            continue

        ssh_endpoint = urllib.parse.urlparse(workspace.endpoints.ssh.endpoint)

        config_value = SSH_CONFIG_FORMAT.format(
            host=host,
            hostname=ssh_endpoint.hostname,
            port=ssh_endpoint.port,
        )
        if private_key_path:
            config_value += f'    IdentityFile {private_key_path}\n'

        with open(SSH_CONFIG_PATH, "a") as f:
            f.write(config_value)



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class DashboardChartCreateAPIPayload(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'field_ids': 'list[int]',
        'ignore_outliers': 'bool',
        'name': 'str',
        'show_legend': 'bool',
        'show_original': 'bool',
        'smoothing': 'float',
        'x_axis': 'str',
        'x_axis_max': 'float',
        'x_axis_min': 'float',
        'x_axis_name': 'str',
        'y_axis_max': 'float',
        'y_axis_min': 'float',
        'y_axis_name': 'str'
    }

    attribute_map = {
        'field_ids': 'field_ids',
        'ignore_outliers': 'ignore_outliers',
        'name': 'name',
        'show_legend': 'show_legend',
        'show_original': 'show_original',
        'smoothing': 'smoothing',
        'x_axis': 'x_axis',
        'x_axis_max': 'x_axis_max',
        'x_axis_min': 'x_axis_min',
        'x_axis_name': 'x_axis_name',
        'y_axis_max': 'y_axis_max',
        'y_axis_min': 'y_axis_min',
        'y_axis_name': 'y_axis_name'
    }

    def __init__(self, field_ids=None, ignore_outliers=None, name=None, show_legend=None, show_original=None, smoothing=None, x_axis=None, x_axis_max=None, x_axis_min=None, x_axis_name=None, y_axis_max=None, y_axis_min=None, y_axis_name=None, local_vars_configuration=None):  # noqa: E501
        """DashboardChartCreateAPIPayload - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._field_ids = None
        self._ignore_outliers = None
        self._name = None
        self._show_legend = None
        self._show_original = None
        self._smoothing = None
        self._x_axis = None
        self._x_axis_max = None
        self._x_axis_min = None
        self._x_axis_name = None
        self._y_axis_max = None
        self._y_axis_min = None
        self._y_axis_name = None
        self.discriminator = None

        if field_ids is not None:
            self.field_ids = field_ids
        self.ignore_outliers = ignore_outliers
        self.name = name
        self.show_legend = show_legend
        self.show_original = show_original
        self.smoothing = smoothing
        self.x_axis = x_axis
        self.x_axis_max = x_axis_max
        self.x_axis_min = x_axis_min
        self.x_axis_name = x_axis_name
        self.y_axis_max = y_axis_max
        self.y_axis_min = y_axis_min
        self.y_axis_name = y_axis_name

    @property
    def field_ids(self):
        """Gets the field_ids of this DashboardChartCreateAPIPayload.  # noqa: E501


        :return: The field_ids of this DashboardChartCreateAPIPayload.  # noqa: E501
        :rtype: list[int]
        """
        return self._field_ids

    @field_ids.setter
    def field_ids(self, field_ids):
        """Sets the field_ids of this DashboardChartCreateAPIPayload.


        :param field_ids: The field_ids of this DashboardChartCreateAPIPayload.  # noqa: E501
        :type field_ids: list[int]
        """

        self._field_ids = field_ids

    @property
    def ignore_outliers(self):
        """Gets the ignore_outliers of this DashboardChartCreateAPIPayload.  # noqa: E501


        :return: The ignore_outliers of this DashboardChartCreateAPIPayload.  # noqa: E501
        :rtype: bool
        """
        return self._ignore_outliers

    @ignore_outliers.setter
    def ignore_outliers(self, ignore_outliers):
        """Sets the ignore_outliers of this DashboardChartCreateAPIPayload.


        :param ignore_outliers: The ignore_outliers of this DashboardChartCreateAPIPayload.  # noqa: E501
        :type ignore_outliers: bool
        """

        self._ignore_outliers = ignore_outliers

    @property
    def name(self):
        """Gets the name of this DashboardChartCreateAPIPayload.  # noqa: E501


        :return: The name of this DashboardChartCreateAPIPayload.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this DashboardChartCreateAPIPayload.


        :param name: The name of this DashboardChartCreateAPIPayload.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def show_legend(self):
        """Gets the show_legend of this DashboardChartCreateAPIPayload.  # noqa: E501


        :return: The show_legend of this DashboardChartCreateAPIPayload.  # noqa: E501
        :rtype: bool
        """
        return self._show_legend

    @show_legend.setter
    def show_legend(self, show_legend):
        """Sets the show_legend of this DashboardChartCreateAPIPayload.


        :param show_legend: The show_legend of this DashboardChartCreateAPIPayload.  # noqa: E501
        :type show_legend: bool
        """

        self._show_legend = show_legend

    @property
    def show_original(self):
        """Gets the show_original of this DashboardChartCreateAPIPayload.  # noqa: E501


        :return: The show_original of this DashboardChartCreateAPIPayload.  # noqa: E501
        :rtype: bool
        """
        return self._show_original

    @show_original.setter
    def show_original(self, show_original):
        """Sets the show_original of this DashboardChartCreateAPIPayload.


        :param show_original: The show_original of this DashboardChartCreateAPIPayload.  # noqa: E501
        :type show_original: bool
        """

        self._show_original = show_original

    @property
    def smoothing(self):
        """Gets the smoothing of this DashboardChartCreateAPIPayload.  # noqa: E501


        :return: The smoothing of this DashboardChartCreateAPIPayload.  # noqa: E501
        :rtype: float
        """
        return self._smoothing

    @smoothing.setter
    def smoothing(self, smoothing):
        """Sets the smoothing of this DashboardChartCreateAPIPayload.


        :param smoothing: The smoothing of this DashboardChartCreateAPIPayload.  # noqa: E501
        :type smoothing: float
        """

        self._smoothing = smoothing

    @property
    def x_axis(self):
        """Gets the x_axis of this DashboardChartCreateAPIPayload.  # noqa: E501


        :return: The x_axis of this DashboardChartCreateAPIPayload.  # noqa: E501
        :rtype: str
        """
        return self._x_axis

    @x_axis.setter
    def x_axis(self, x_axis):
        """Sets the x_axis of this DashboardChartCreateAPIPayload.


        :param x_axis: The x_axis of this DashboardChartCreateAPIPayload.  # noqa: E501
        :type x_axis: str
        """

        self._x_axis = x_axis

    @property
    def x_axis_max(self):
        """Gets the x_axis_max of this DashboardChartCreateAPIPayload.  # noqa: E501


        :return: The x_axis_max of this DashboardChartCreateAPIPayload.  # noqa: E501
        :rtype: float
        """
        return self._x_axis_max

    @x_axis_max.setter
    def x_axis_max(self, x_axis_max):
        """Sets the x_axis_max of this DashboardChartCreateAPIPayload.


        :param x_axis_max: The x_axis_max of this DashboardChartCreateAPIPayload.  # noqa: E501
        :type x_axis_max: float
        """

        self._x_axis_max = x_axis_max

    @property
    def x_axis_min(self):
        """Gets the x_axis_min of this DashboardChartCreateAPIPayload.  # noqa: E501


        :return: The x_axis_min of this DashboardChartCreateAPIPayload.  # noqa: E501
        :rtype: float
        """
        return self._x_axis_min

    @x_axis_min.setter
    def x_axis_min(self, x_axis_min):
        """Sets the x_axis_min of this DashboardChartCreateAPIPayload.


        :param x_axis_min: The x_axis_min of this DashboardChartCreateAPIPayload.  # noqa: E501
        :type x_axis_min: float
        """

        self._x_axis_min = x_axis_min

    @property
    def x_axis_name(self):
        """Gets the x_axis_name of this DashboardChartCreateAPIPayload.  # noqa: E501


        :return: The x_axis_name of this DashboardChartCreateAPIPayload.  # noqa: E501
        :rtype: str
        """
        return self._x_axis_name

    @x_axis_name.setter
    def x_axis_name(self, x_axis_name):
        """Sets the x_axis_name of this DashboardChartCreateAPIPayload.


        :param x_axis_name: The x_axis_name of this DashboardChartCreateAPIPayload.  # noqa: E501
        :type x_axis_name: str
        """

        self._x_axis_name = x_axis_name

    @property
    def y_axis_max(self):
        """Gets the y_axis_max of this DashboardChartCreateAPIPayload.  # noqa: E501


        :return: The y_axis_max of this DashboardChartCreateAPIPayload.  # noqa: E501
        :rtype: float
        """
        return self._y_axis_max

    @y_axis_max.setter
    def y_axis_max(self, y_axis_max):
        """Sets the y_axis_max of this DashboardChartCreateAPIPayload.


        :param y_axis_max: The y_axis_max of this DashboardChartCreateAPIPayload.  # noqa: E501
        :type y_axis_max: float
        """

        self._y_axis_max = y_axis_max

    @property
    def y_axis_min(self):
        """Gets the y_axis_min of this DashboardChartCreateAPIPayload.  # noqa: E501


        :return: The y_axis_min of this DashboardChartCreateAPIPayload.  # noqa: E501
        :rtype: float
        """
        return self._y_axis_min

    @y_axis_min.setter
    def y_axis_min(self, y_axis_min):
        """Sets the y_axis_min of this DashboardChartCreateAPIPayload.


        :param y_axis_min: The y_axis_min of this DashboardChartCreateAPIPayload.  # noqa: E501
        :type y_axis_min: float
        """

        self._y_axis_min = y_axis_min

    @property
    def y_axis_name(self):
        """Gets the y_axis_name of this DashboardChartCreateAPIPayload.  # noqa: E501


        :return: The y_axis_name of this DashboardChartCreateAPIPayload.  # noqa: E501
        :rtype: str
        """
        return self._y_axis_name

    @y_axis_name.setter
    def y_axis_name(self, y_axis_name):
        """Sets the y_axis_name of this DashboardChartCreateAPIPayload.


        :param y_axis_name: The y_axis_name of this DashboardChartCreateAPIPayload.  # noqa: E501
        :type y_axis_name: str
        """

        self._y_axis_name = y_axis_name

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DashboardChartCreateAPIPayload):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, DashboardChartCreateAPIPayload):
            return True

        return self.to_dict() != other.to_dict()



from typing import Dict, List, Tuple

from openapi_client.models import (
    CliExperimentPlotsFilesUpdateAPIPayload,
    CliExperimentPlotsMetricsUpdateAPIPayload,
    ExperimentCreateAPIPayload,
    InfluxdbExperimentPlotFile,
    InfluxdbExperimentPlotMetric,
    InfluxdbWorkloadLog,
    OrmEnvVar,
    ResponseExperimentInfo,
    ResponseExperimentListResponse,
    ResponseFileMetadata,
)
from vessl import vessl_api
from vessl.kernel_cluster import read_cluster
from vessl.kernel_resource_spec import (
    _configure_custom_kernel_resource_spec,
    read_kernel_resource_spec,
)
from vessl.organization import _get_organization_name
from vessl.project import _get_project_name
from vessl.util import logger
from vessl.util.constant import MOUNT_PATH_OUTPUT, MOUNT_TYPE_OUTPUT
from vessl.volume import (
    _configure_volume_mount_requests,
    copy_volume_file,
    list_volume_files,
)


def read_experiment(experiment_name_or_number: str, **kwargs) -> ResponseExperimentInfo:
    """Read experiment

    Keyword args:
        organization_name (str): override default organization
        project_name (str): override default project
    """
    return vessl_api.experiment_read_api(
        experiment=experiment_name_or_number,
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
    )


def read_experiment_by_id(experiment_id: int, **kwargs) -> ResponseExperimentInfo:
    return vessl_api.experiment_read_by_idapi(experiment_id=experiment_id)


def list_experiments(**kwargs) -> List[ResponseExperimentListResponse]:
    """List experiments

    Keyword args:
        organization_name (str): override default organization
        project_name (str): override default project
    """
    return vessl_api.experiment_list_api(
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
    ).results


def create_experiment(
    cluster_name: str,
    start_command: str,
    kernel_resource_spec_name: str = None,
    processor_type: str = None,
    cpu_limit: float = None,
    memory_limit: float = None,
    gpu_type: str = None,
    gpu_limit: int = None,
    kernel_image_url: str = None,
    *,
    message: str = None,
    termination_protection: bool = False,
    env_vars: List[Tuple[str, str]] = None,
    dataset_mounts: List[Tuple[str, str]] = None,
    root_volume_size: str = None,
    working_dir: str = None,
    output_dir: str = MOUNT_PATH_OUTPUT,
    local_project_url: str = None,
    **kwargs,
) -> ResponseExperimentInfo:
    """Create experiment

    Keyword args:
        organization_name (str): override default organization
        project_name (str): override default project
        git_branch (str): override current git branch
        git_ref (str): override current git commit
        use_git_diff (bool): run experiment with uncommitted changes
        use_git_diff_untracked (bool): run with untracked changed (only valid if `use_git_diff` is set)
    """
    cluster = read_cluster(cluster_name)

    kernel_resource_spec = kernel_resource_spec_id = None
    if cluster.is_savvihub_managed:  # TODO: rename to vessl
        kernel_resource_spec_id = read_kernel_resource_spec(
            kernel_resource_spec_name
        ).id
    else:
        kernel_resource_spec = _configure_custom_kernel_resource_spec(
            processor_type,
            cpu_limit,
            memory_limit,
            gpu_type,
            gpu_limit,
        )

    volume_mount_requests = _configure_volume_mount_requests(
        dataset_mounts=dataset_mounts,
        root_volume_size=root_volume_size,
        working_dir=working_dir,
        output_dir=output_dir,
        local_project_url=local_project_url,
        **kwargs,
    )

    return vessl_api.experiment_create_api(
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
        experiment_create_api_payload=ExperimentCreateAPIPayload(
            cluster_name=cluster_name,
            env_vars=[OrmEnvVar(key, str(value)) for key, value in (env_vars or [])],
            image_url=kernel_image_url,
            message=message,
            resource_spec=kernel_resource_spec,
            resource_spec_id=kernel_resource_spec_id,
            start_command=start_command,
            termination_protection=termination_protection,
            volumes=volume_mount_requests,
        ),
    )


def list_experiment_logs(
    experiment_name: str, tail: int = 200, **kwargs
) -> List[InfluxdbWorkloadLog]:
    """List experiment logs

    Args:
        tail (int): number of lines to display from the end. Display all if -1.

    Keyword args:
        organization_name (str): override default organization
        project_name (str): override default project
    """
    if tail == -1:
        tail = None

    return vessl_api.experiment_logs_api(
        experiment=experiment_name,
        limit=tail,
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
    ).logs


def list_experiment_output_files(
    experiment_name: str,
    need_download_url: bool = False,
    recursive: bool = False,
    **kwargs,
) -> List[ResponseFileMetadata]:
    """List experiment output files

    Keyword args:
        organization_name (str): override default organization
        project_name (str): override default project
    """
    experiment_name = read_experiment(experiment_name, **kwargs)

    for volume_mount in experiment_name.volume_mounts.mounts:
        if volume_mount.source_type == MOUNT_TYPE_OUTPUT:
            return list_volume_files(
                volume_id=volume_mount.volume.volume_id,
                need_download_url=need_download_url,
                path="",
                recursive=recursive,
            )

    logger.info("No output volume mounted")


def download_experiment_output_files(
    experiment_name: str, dest_path: str = "./output", **kwargs
) -> None:
    """Download experiment output files

    Keyword args:
        organization_name (str): override default organization
        project_name (str): override default project
    """
    experiment = read_experiment(experiment_name, **kwargs)
    for volume_mount in experiment.volume_mounts.mounts:
        if volume_mount.source_type == MOUNT_TYPE_OUTPUT:
            return copy_volume_file(
                source_volume_id=volume_mount.volume.volume_id,
                source_path="",
                dest_volume_id=None,
                dest_path=dest_path,
                recursive=True,
            )

    logger.info("No output volume mounted")


def update_experiment_plots_files(
    experiment_id: int, files: List[InfluxdbExperimentPlotFile], type: str
) -> object:
    return vessl_api.cli_experiment_plots_files_update_api(
        experiment_id=experiment_id,
        cli_experiment_plots_files_update_api_payload=CliExperimentPlotsFilesUpdateAPIPayload(
            files=files,
            type=type,
        ),
    )


def update_experiment_plots_metrics(
    experiment_id: int, workload_id: int, metrics: Dict[str, List[InfluxdbExperimentPlotMetric]]
) -> object:
    return vessl_api.cli_experiment_plots_metrics_update_api(
        experiment_id=experiment_id,
        cli_experiment_plots_metrics_update_api_payload=CliExperimentPlotsMetricsUpdateAPIPayload(
            metrics=metrics,
            workload_id=workload_id,
        ),
    )



import sys
from typing import Any, Callable, List, Optional

import click
import sentry_sdk

import vessl
from vessl.util import logger
from vessl.util.constant import VESSL_LOG_LEVEL
from vessl.util.exception import (
    DEFAULT_ERROR_MESSAGE,
    InvalidTokenError,
    SavvihubException,
)


def no_interaction_callback(ctx: click.Context, param: click.Parameter, value: bool):
    ctx.obj["no_interaction"] = value


no_interaction_option = click.Option(
    ["--no-interaction"],
    is_flag=True,
    is_eager=True,
    expose_value=False,
    hidden=True,
    callback=no_interaction_callback,
)


class VesslArgument(click.Argument):
    """Custom argument class"""

    def __init__(self, *args, **kwargs) -> None:
        self.prompter = kwargs.pop("prompter", None)
        super().__init__(*args, **kwargs)

    def process_value(self, ctx: click.Context, value: Any) -> Any:
        """Override parent method to invoke prompter before processing value."""
        if (
            (value is None or (self.multiple == True and not value))
            and self.prompter is not None
            and not ctx.obj["no_interaction"]
        ):
            value = self.prompter(ctx, self, value)

        return super().process_value(ctx, value)


class VesslOption(click.Option):
    """Custom option class"""

    def __init__(self, *args, **kwargs) -> None:
        self.prompter = kwargs.pop("prompter", None)
        super().__init__(*args, **kwargs)

    def process_value(self, ctx: click.Context, value: Any) -> Any:
        """Override parent method to invoke prompter before processing value."""
        if (
            (value is None or (self.multiple == True and not value))
            and self.prompter is not None
            and not ctx.obj["no_interaction"]
        ):
            value = self.prompter(ctx, self, value)

        return super().process_value(ctx, value)


class VesslCommand(click.Command):
    def __init__(
        self,
        login_required: bool = False,
        params: Optional[List[click.Parameter]] = None,
        **kwargs,
    ) -> None:
        self.login_required = login_required

        params = params or []
        params = [no_interaction_option] + params

        super().__init__(params=params, **kwargs)

    def make_context(self, *args, **kwargs) -> click.Context:
        if self.login_required:
            vessl.vessl_api.set_access_token(no_prompt=True)

        return super().make_context(*args, **kwargs)


class VesslGroup(click.Group):
    command_class = VesslCommand

    def vessl_command(self, login_required: bool = True, *args, **kwargs):
        kwargs["login_required"] = login_required
        return super().command(*args, **kwargs)

    def main(self, *args, **kwargs):
        try:
            return super().main(*args, **kwargs)

        except InvalidTokenError:
            print("Invalid access token. Please run `vessl configure`.")
            return

        except SavvihubException as e:
            exc_info = e if VESSL_LOG_LEVEL == "DEBUG" else False
            logger.exception(f"{e.__class__.__name__}: {str(e)}", exc_info=exc_info)
            sys.exit(e.exit_code)

        except Exception as e:
            sentry_sdk.capture_exception(e)
            sentry_sdk.flush()

            exc_info = e if VESSL_LOG_LEVEL == "DEBUG" else False
            logger.fatal(DEFAULT_ERROR_MESSAGE, exc_info=exc_info)
            sys.exit(1)


def vessl_argument(*param_decls: str, **attrs: Any) -> Callable:
    attrs["cls"] = VesslArgument
    return click.argument(*param_decls, **attrs)


def vessl_option(*param_decls: str, **attrs: Any) -> Callable:
    attrs["cls"] = VesslOption
    return click.option(*param_decls, **attrs)



from test.conftest import USE_MOCK

import pytest

import vessl


@pytest.mark.skipif(USE_MOCK, reason="Does not run if mocking is used.")
class TestKernelResourceSpec:
    def test_read_kernel_resource_spec(self):
        vessl.read_kernel_resource_spec("v1.cpu-0.mem-1")

    def test_list_kernel_resource_specs(self):
        vessl.list_kernel_images()



from datetime import datetime
from test.conftest import USE_MOCK

import pytest

import vessl
from openapi_client.models import (
    InfluxdbExperimentPlotFile,
    InfluxdbExperimentPlotMetric,
)
from vessl.util.constant import VESSL_PLOTS_FILETYPE_IMAGE


@pytest.mark.skipif(USE_MOCK, reason="Does not run if mocking is used.")
class TestExperiment:
    @pytest.mark.order(index=1)
    def test_create_experiment(self):
        # Use pytest.experiment instead of self.experiment because self gets reset
        # after each test
        pytest.experiment = vessl.create_experiment(
            cluster_name="aws-uw2-dev",  # TODO: change when CI server is relocated
            start_command="echo 'Hello world!'",
            kernel_resource_spec_name="v1.cpu-0.mem-1",
            kernel_image_url="vessl/kernels:py36.full-cpu",
            message="test message",
            termination_protection=False,
            env_vars=[("VAR", "value")],
            local_project_url="",
        )

    def test_read_experiment(self):
        vessl.read_experiment(pytest.experiment.name)

    def test_read_experiment_by_id(self):
        vessl.read_experiment_by_id(pytest.experiment.id)

    def test_list_experiments(self):
        vessl.list_experiments()

    def test_list_experiment_logs(self):
        vessl.list_experiment_logs(pytest.experiment.name)

    def test_list_experiment_output_files(self):
        vessl.list_experiment_output_files(pytest.experiment.name)

    def test_download_experiment_output_files(self):
        vessl.download_experiment_output_files(pytest.experiment.name)

    def test_update_experiment_plots_files(self):
        file = InfluxdbExperimentPlotFile(
            step=None,
            path="test/fixture/file1.txt",
            caption="test-caption",
            timestamp=datetime.utcnow().timestamp(),
        )
        vessl.update_experiment_plots_files(
            pytest.experiment.id, [file], VESSL_PLOTS_FILETYPE_IMAGE
        )

    def test_update_experiment_plots_metrics(self):
        metric = InfluxdbExperimentPlotMetric(
            step=1,
            timestamp=datetime.utcnow().timestamp(),
            value=1.0,
        )
        vessl.update_experiment_plots_metrics(
            pytest.experiment.id, pytest.experiment.workload_id, {"val_accuracy": [metric]}
        )



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponseVolume(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'bucket_name': 'str',
        'created_dt': 'datetime',
        'file_count': 'int',
        'id': 'int',
        'is_read_only': 'bool',
        'last_sync_time': 'datetime',
        'local_volume_config': 'OrmLocalVolumeConfig',
        'size': 'int',
        'status': 'str',
        'updated_dt': 'datetime'
    }

    attribute_map = {
        'bucket_name': 'bucket_name',
        'created_dt': 'created_dt',
        'file_count': 'file_count',
        'id': 'id',
        'is_read_only': 'is_read_only',
        'last_sync_time': 'last_sync_time',
        'local_volume_config': 'local_volume_config',
        'size': 'size',
        'status': 'status',
        'updated_dt': 'updated_dt'
    }

    def __init__(self, bucket_name=None, created_dt=None, file_count=None, id=None, is_read_only=None, last_sync_time=None, local_volume_config=None, size=None, status=None, updated_dt=None, local_vars_configuration=None):  # noqa: E501
        """ResponseVolume - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._bucket_name = None
        self._created_dt = None
        self._file_count = None
        self._id = None
        self._is_read_only = None
        self._last_sync_time = None
        self._local_volume_config = None
        self._size = None
        self._status = None
        self._updated_dt = None
        self.discriminator = None

        self.bucket_name = bucket_name
        self.created_dt = created_dt
        self.file_count = file_count
        self.id = id
        self.is_read_only = is_read_only
        self.last_sync_time = last_sync_time
        if local_volume_config is not None:
            self.local_volume_config = local_volume_config
        self.size = size
        self.status = status
        self.updated_dt = updated_dt

    @property
    def bucket_name(self):
        """Gets the bucket_name of this ResponseVolume.  # noqa: E501


        :return: The bucket_name of this ResponseVolume.  # noqa: E501
        :rtype: str
        """
        return self._bucket_name

    @bucket_name.setter
    def bucket_name(self, bucket_name):
        """Sets the bucket_name of this ResponseVolume.


        :param bucket_name: The bucket_name of this ResponseVolume.  # noqa: E501
        :type bucket_name: str
        """

        self._bucket_name = bucket_name

    @property
    def created_dt(self):
        """Gets the created_dt of this ResponseVolume.  # noqa: E501


        :return: The created_dt of this ResponseVolume.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ResponseVolume.


        :param created_dt: The created_dt of this ResponseVolume.  # noqa: E501
        :type created_dt: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_dt is None:  # noqa: E501
            raise ValueError("Invalid value for `created_dt`, must not be `None`")  # noqa: E501

        self._created_dt = created_dt

    @property
    def file_count(self):
        """Gets the file_count of this ResponseVolume.  # noqa: E501


        :return: The file_count of this ResponseVolume.  # noqa: E501
        :rtype: int
        """
        return self._file_count

    @file_count.setter
    def file_count(self, file_count):
        """Sets the file_count of this ResponseVolume.


        :param file_count: The file_count of this ResponseVolume.  # noqa: E501
        :type file_count: int
        """
        if self.local_vars_configuration.client_side_validation and file_count is None:  # noqa: E501
            raise ValueError("Invalid value for `file_count`, must not be `None`")  # noqa: E501

        self._file_count = file_count

    @property
    def id(self):
        """Gets the id of this ResponseVolume.  # noqa: E501


        :return: The id of this ResponseVolume.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ResponseVolume.


        :param id: The id of this ResponseVolume.  # noqa: E501
        :type id: int
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def is_read_only(self):
        """Gets the is_read_only of this ResponseVolume.  # noqa: E501


        :return: The is_read_only of this ResponseVolume.  # noqa: E501
        :rtype: bool
        """
        return self._is_read_only

    @is_read_only.setter
    def is_read_only(self, is_read_only):
        """Sets the is_read_only of this ResponseVolume.


        :param is_read_only: The is_read_only of this ResponseVolume.  # noqa: E501
        :type is_read_only: bool
        """
        if self.local_vars_configuration.client_side_validation and is_read_only is None:  # noqa: E501
            raise ValueError("Invalid value for `is_read_only`, must not be `None`")  # noqa: E501

        self._is_read_only = is_read_only

    @property
    def last_sync_time(self):
        """Gets the last_sync_time of this ResponseVolume.  # noqa: E501


        :return: The last_sync_time of this ResponseVolume.  # noqa: E501
        :rtype: datetime
        """
        return self._last_sync_time

    @last_sync_time.setter
    def last_sync_time(self, last_sync_time):
        """Sets the last_sync_time of this ResponseVolume.


        :param last_sync_time: The last_sync_time of this ResponseVolume.  # noqa: E501
        :type last_sync_time: datetime
        """

        self._last_sync_time = last_sync_time

    @property
    def local_volume_config(self):
        """Gets the local_volume_config of this ResponseVolume.  # noqa: E501


        :return: The local_volume_config of this ResponseVolume.  # noqa: E501
        :rtype: OrmLocalVolumeConfig
        """
        return self._local_volume_config

    @local_volume_config.setter
    def local_volume_config(self, local_volume_config):
        """Sets the local_volume_config of this ResponseVolume.


        :param local_volume_config: The local_volume_config of this ResponseVolume.  # noqa: E501
        :type local_volume_config: OrmLocalVolumeConfig
        """

        self._local_volume_config = local_volume_config

    @property
    def size(self):
        """Gets the size of this ResponseVolume.  # noqa: E501


        :return: The size of this ResponseVolume.  # noqa: E501
        :rtype: int
        """
        return self._size

    @size.setter
    def size(self, size):
        """Sets the size of this ResponseVolume.


        :param size: The size of this ResponseVolume.  # noqa: E501
        :type size: int
        """
        if self.local_vars_configuration.client_side_validation and size is None:  # noqa: E501
            raise ValueError("Invalid value for `size`, must not be `None`")  # noqa: E501

        self._size = size

    @property
    def status(self):
        """Gets the status of this ResponseVolume.  # noqa: E501


        :return: The status of this ResponseVolume.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this ResponseVolume.


        :param status: The status of this ResponseVolume.  # noqa: E501
        :type status: str
        """
        if self.local_vars_configuration.client_side_validation and status is None:  # noqa: E501
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501

        self._status = status

    @property
    def updated_dt(self):
        """Gets the updated_dt of this ResponseVolume.  # noqa: E501


        :return: The updated_dt of this ResponseVolume.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this ResponseVolume.


        :param updated_dt: The updated_dt of this ResponseVolume.  # noqa: E501
        :type updated_dt: datetime
        """
        if self.local_vars_configuration.client_side_validation and updated_dt is None:  # noqa: E501
            raise ValueError("Invalid value for `updated_dt`, must not be `None`")  # noqa: E501

        self._updated_dt = updated_dt

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseVolume):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseVolume):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class GithubRepository(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'allow_auto_merge': 'bool',
        'allow_merge_commit': 'bool',
        'allow_rebase_merge': 'bool',
        'allow_squash_merge': 'bool',
        'archive_url': 'str',
        'archived': 'bool',
        'assignees_url': 'str',
        'auto_init': 'bool',
        'blobs_url': 'str',
        'branches_url': 'str',
        'clone_url': 'str',
        'code_of_conduct': 'GithubCodeOfConduct',
        'collaborators_url': 'str',
        'comments_url': 'str',
        'commits_url': 'str',
        'compare_url': 'str',
        'contents_url': 'str',
        'contributors_url': 'str',
        'created_at': 'object',
        'default_branch': 'str',
        'delete_branch_on_merge': 'bool',
        'deployments_url': 'str',
        'description': 'str',
        'disabled': 'bool',
        'downloads_url': 'str',
        'events_url': 'str',
        'fork': 'bool',
        'forks_count': 'int',
        'forks_url': 'str',
        'full_name': 'str',
        'git_commits_url': 'str',
        'git_refs_url': 'str',
        'git_tags_url': 'str',
        'git_url': 'str',
        'gitignore_template': 'str',
        'has_downloads': 'bool',
        'has_issues': 'bool',
        'has_pages': 'bool',
        'has_projects': 'bool',
        'has_wiki': 'bool',
        'homepage': 'str',
        'hooks_url': 'str',
        'html_url': 'str',
        'id': 'int',
        'is_template': 'bool',
        'issue_comment_url': 'str',
        'issue_events_url': 'str',
        'issues_url': 'str',
        'keys_url': 'str',
        'labels_url': 'str',
        'language': 'str',
        'languages_url': 'str',
        'license': 'GithubLicense',
        'license_template': 'str',
        'master_branch': 'str',
        'merges_url': 'str',
        'milestones_url': 'str',
        'mirror_url': 'str',
        'name': 'str',
        'network_count': 'int',
        'node_id': 'str',
        'notifications_url': 'str',
        'open_issues': 'int',
        'open_issues_count': 'int',
        'organization': 'GithubOrganization',
        'owner': 'GithubUser',
        'parent': 'GithubRepository',
        'permissions': 'dict(str, bool)',
        'private': 'bool',
        'pulls_url': 'str',
        'pushed_at': 'object',
        'releases_url': 'str',
        'size': 'int',
        'source': 'GithubRepository',
        'ssh_url': 'str',
        'stargazers_count': 'int',
        'stargazers_url': 'str',
        'statuses_url': 'str',
        'subscribers_count': 'int',
        'subscribers_url': 'str',
        'subscription_url': 'str',
        'svn_url': 'str',
        'tags_url': 'str',
        'team_id': 'int',
        'teams_url': 'str',
        'template_repository': 'GithubRepository',
        'text_matches': 'list[GithubTextMatch]',
        'topics': 'list[str]',
        'trees_url': 'str',
        'updated_at': 'object',
        'url': 'str',
        'visibility': 'str',
        'watchers': 'int',
        'watchers_count': 'int'
    }

    attribute_map = {
        'allow_auto_merge': 'allow_auto_merge',
        'allow_merge_commit': 'allow_merge_commit',
        'allow_rebase_merge': 'allow_rebase_merge',
        'allow_squash_merge': 'allow_squash_merge',
        'archive_url': 'archive_url',
        'archived': 'archived',
        'assignees_url': 'assignees_url',
        'auto_init': 'auto_init',
        'blobs_url': 'blobs_url',
        'branches_url': 'branches_url',
        'clone_url': 'clone_url',
        'code_of_conduct': 'code_of_conduct',
        'collaborators_url': 'collaborators_url',
        'comments_url': 'comments_url',
        'commits_url': 'commits_url',
        'compare_url': 'compare_url',
        'contents_url': 'contents_url',
        'contributors_url': 'contributors_url',
        'created_at': 'created_at',
        'default_branch': 'default_branch',
        'delete_branch_on_merge': 'delete_branch_on_merge',
        'deployments_url': 'deployments_url',
        'description': 'description',
        'disabled': 'disabled',
        'downloads_url': 'downloads_url',
        'events_url': 'events_url',
        'fork': 'fork',
        'forks_count': 'forks_count',
        'forks_url': 'forks_url',
        'full_name': 'full_name',
        'git_commits_url': 'git_commits_url',
        'git_refs_url': 'git_refs_url',
        'git_tags_url': 'git_tags_url',
        'git_url': 'git_url',
        'gitignore_template': 'gitignore_template',
        'has_downloads': 'has_downloads',
        'has_issues': 'has_issues',
        'has_pages': 'has_pages',
        'has_projects': 'has_projects',
        'has_wiki': 'has_wiki',
        'homepage': 'homepage',
        'hooks_url': 'hooks_url',
        'html_url': 'html_url',
        'id': 'id',
        'is_template': 'is_template',
        'issue_comment_url': 'issue_comment_url',
        'issue_events_url': 'issue_events_url',
        'issues_url': 'issues_url',
        'keys_url': 'keys_url',
        'labels_url': 'labels_url',
        'language': 'language',
        'languages_url': 'languages_url',
        'license': 'license',
        'license_template': 'license_template',
        'master_branch': 'master_branch',
        'merges_url': 'merges_url',
        'milestones_url': 'milestones_url',
        'mirror_url': 'mirror_url',
        'name': 'name',
        'network_count': 'network_count',
        'node_id': 'node_id',
        'notifications_url': 'notifications_url',
        'open_issues': 'open_issues',
        'open_issues_count': 'open_issues_count',
        'organization': 'organization',
        'owner': 'owner',
        'parent': 'parent',
        'permissions': 'permissions',
        'private': 'private',
        'pulls_url': 'pulls_url',
        'pushed_at': 'pushed_at',
        'releases_url': 'releases_url',
        'size': 'size',
        'source': 'source',
        'ssh_url': 'ssh_url',
        'stargazers_count': 'stargazers_count',
        'stargazers_url': 'stargazers_url',
        'statuses_url': 'statuses_url',
        'subscribers_count': 'subscribers_count',
        'subscribers_url': 'subscribers_url',
        'subscription_url': 'subscription_url',
        'svn_url': 'svn_url',
        'tags_url': 'tags_url',
        'team_id': 'team_id',
        'teams_url': 'teams_url',
        'template_repository': 'template_repository',
        'text_matches': 'text_matches',
        'topics': 'topics',
        'trees_url': 'trees_url',
        'updated_at': 'updated_at',
        'url': 'url',
        'visibility': 'visibility',
        'watchers': 'watchers',
        'watchers_count': 'watchers_count'
    }

    def __init__(self, allow_auto_merge=None, allow_merge_commit=None, allow_rebase_merge=None, allow_squash_merge=None, archive_url=None, archived=None, assignees_url=None, auto_init=None, blobs_url=None, branches_url=None, clone_url=None, code_of_conduct=None, collaborators_url=None, comments_url=None, commits_url=None, compare_url=None, contents_url=None, contributors_url=None, created_at=None, default_branch=None, delete_branch_on_merge=None, deployments_url=None, description=None, disabled=None, downloads_url=None, events_url=None, fork=None, forks_count=None, forks_url=None, full_name=None, git_commits_url=None, git_refs_url=None, git_tags_url=None, git_url=None, gitignore_template=None, has_downloads=None, has_issues=None, has_pages=None, has_projects=None, has_wiki=None, homepage=None, hooks_url=None, html_url=None, id=None, is_template=None, issue_comment_url=None, issue_events_url=None, issues_url=None, keys_url=None, labels_url=None, language=None, languages_url=None, license=None, license_template=None, master_branch=None, merges_url=None, milestones_url=None, mirror_url=None, name=None, network_count=None, node_id=None, notifications_url=None, open_issues=None, open_issues_count=None, organization=None, owner=None, parent=None, permissions=None, private=None, pulls_url=None, pushed_at=None, releases_url=None, size=None, source=None, ssh_url=None, stargazers_count=None, stargazers_url=None, statuses_url=None, subscribers_count=None, subscribers_url=None, subscription_url=None, svn_url=None, tags_url=None, team_id=None, teams_url=None, template_repository=None, text_matches=None, topics=None, trees_url=None, updated_at=None, url=None, visibility=None, watchers=None, watchers_count=None, local_vars_configuration=None):  # noqa: E501
        """GithubRepository - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._allow_auto_merge = None
        self._allow_merge_commit = None
        self._allow_rebase_merge = None
        self._allow_squash_merge = None
        self._archive_url = None
        self._archived = None
        self._assignees_url = None
        self._auto_init = None
        self._blobs_url = None
        self._branches_url = None
        self._clone_url = None
        self._code_of_conduct = None
        self._collaborators_url = None
        self._comments_url = None
        self._commits_url = None
        self._compare_url = None
        self._contents_url = None
        self._contributors_url = None
        self._created_at = None
        self._default_branch = None
        self._delete_branch_on_merge = None
        self._deployments_url = None
        self._description = None
        self._disabled = None
        self._downloads_url = None
        self._events_url = None
        self._fork = None
        self._forks_count = None
        self._forks_url = None
        self._full_name = None
        self._git_commits_url = None
        self._git_refs_url = None
        self._git_tags_url = None
        self._git_url = None
        self._gitignore_template = None
        self._has_downloads = None
        self._has_issues = None
        self._has_pages = None
        self._has_projects = None
        self._has_wiki = None
        self._homepage = None
        self._hooks_url = None
        self._html_url = None
        self._id = None
        self._is_template = None
        self._issue_comment_url = None
        self._issue_events_url = None
        self._issues_url = None
        self._keys_url = None
        self._labels_url = None
        self._language = None
        self._languages_url = None
        self._license = None
        self._license_template = None
        self._master_branch = None
        self._merges_url = None
        self._milestones_url = None
        self._mirror_url = None
        self._name = None
        self._network_count = None
        self._node_id = None
        self._notifications_url = None
        self._open_issues = None
        self._open_issues_count = None
        self._organization = None
        self._owner = None
        self._parent = None
        self._permissions = None
        self._private = None
        self._pulls_url = None
        self._pushed_at = None
        self._releases_url = None
        self._size = None
        self._source = None
        self._ssh_url = None
        self._stargazers_count = None
        self._stargazers_url = None
        self._statuses_url = None
        self._subscribers_count = None
        self._subscribers_url = None
        self._subscription_url = None
        self._svn_url = None
        self._tags_url = None
        self._team_id = None
        self._teams_url = None
        self._template_repository = None
        self._text_matches = None
        self._topics = None
        self._trees_url = None
        self._updated_at = None
        self._url = None
        self._visibility = None
        self._watchers = None
        self._watchers_count = None
        self.discriminator = None

        self.allow_auto_merge = allow_auto_merge
        self.allow_merge_commit = allow_merge_commit
        self.allow_rebase_merge = allow_rebase_merge
        self.allow_squash_merge = allow_squash_merge
        self.archive_url = archive_url
        self.archived = archived
        self.assignees_url = assignees_url
        self.auto_init = auto_init
        self.blobs_url = blobs_url
        self.branches_url = branches_url
        self.clone_url = clone_url
        if code_of_conduct is not None:
            self.code_of_conduct = code_of_conduct
        self.collaborators_url = collaborators_url
        self.comments_url = comments_url
        self.commits_url = commits_url
        self.compare_url = compare_url
        self.contents_url = contents_url
        self.contributors_url = contributors_url
        if created_at is not None:
            self.created_at = created_at
        self.default_branch = default_branch
        self.delete_branch_on_merge = delete_branch_on_merge
        self.deployments_url = deployments_url
        self.description = description
        self.disabled = disabled
        self.downloads_url = downloads_url
        self.events_url = events_url
        self.fork = fork
        self.forks_count = forks_count
        self.forks_url = forks_url
        self.full_name = full_name
        self.git_commits_url = git_commits_url
        self.git_refs_url = git_refs_url
        self.git_tags_url = git_tags_url
        self.git_url = git_url
        self.gitignore_template = gitignore_template
        self.has_downloads = has_downloads
        self.has_issues = has_issues
        self.has_pages = has_pages
        self.has_projects = has_projects
        self.has_wiki = has_wiki
        self.homepage = homepage
        self.hooks_url = hooks_url
        self.html_url = html_url
        self.id = id
        self.is_template = is_template
        self.issue_comment_url = issue_comment_url
        self.issue_events_url = issue_events_url
        self.issues_url = issues_url
        self.keys_url = keys_url
        self.labels_url = labels_url
        self.language = language
        self.languages_url = languages_url
        if license is not None:
            self.license = license
        self.license_template = license_template
        self.master_branch = master_branch
        self.merges_url = merges_url
        self.milestones_url = milestones_url
        self.mirror_url = mirror_url
        self.name = name
        self.network_count = network_count
        self.node_id = node_id
        self.notifications_url = notifications_url
        self.open_issues = open_issues
        self.open_issues_count = open_issues_count
        if organization is not None:
            self.organization = organization
        if owner is not None:
            self.owner = owner
        if parent is not None:
            self.parent = parent
        if permissions is not None:
            self.permissions = permissions
        self.private = private
        self.pulls_url = pulls_url
        if pushed_at is not None:
            self.pushed_at = pushed_at
        self.releases_url = releases_url
        self.size = size
        if source is not None:
            self.source = source
        self.ssh_url = ssh_url
        self.stargazers_count = stargazers_count
        self.stargazers_url = stargazers_url
        self.statuses_url = statuses_url
        self.subscribers_count = subscribers_count
        self.subscribers_url = subscribers_url
        self.subscription_url = subscription_url
        self.svn_url = svn_url
        self.tags_url = tags_url
        self.team_id = team_id
        self.teams_url = teams_url
        if template_repository is not None:
            self.template_repository = template_repository
        if text_matches is not None:
            self.text_matches = text_matches
        if topics is not None:
            self.topics = topics
        self.trees_url = trees_url
        if updated_at is not None:
            self.updated_at = updated_at
        self.url = url
        self.visibility = visibility
        self.watchers = watchers
        self.watchers_count = watchers_count

    @property
    def allow_auto_merge(self):
        """Gets the allow_auto_merge of this GithubRepository.  # noqa: E501


        :return: The allow_auto_merge of this GithubRepository.  # noqa: E501
        :rtype: bool
        """
        return self._allow_auto_merge

    @allow_auto_merge.setter
    def allow_auto_merge(self, allow_auto_merge):
        """Sets the allow_auto_merge of this GithubRepository.


        :param allow_auto_merge: The allow_auto_merge of this GithubRepository.  # noqa: E501
        :type allow_auto_merge: bool
        """

        self._allow_auto_merge = allow_auto_merge

    @property
    def allow_merge_commit(self):
        """Gets the allow_merge_commit of this GithubRepository.  # noqa: E501


        :return: The allow_merge_commit of this GithubRepository.  # noqa: E501
        :rtype: bool
        """
        return self._allow_merge_commit

    @allow_merge_commit.setter
    def allow_merge_commit(self, allow_merge_commit):
        """Sets the allow_merge_commit of this GithubRepository.


        :param allow_merge_commit: The allow_merge_commit of this GithubRepository.  # noqa: E501
        :type allow_merge_commit: bool
        """

        self._allow_merge_commit = allow_merge_commit

    @property
    def allow_rebase_merge(self):
        """Gets the allow_rebase_merge of this GithubRepository.  # noqa: E501


        :return: The allow_rebase_merge of this GithubRepository.  # noqa: E501
        :rtype: bool
        """
        return self._allow_rebase_merge

    @allow_rebase_merge.setter
    def allow_rebase_merge(self, allow_rebase_merge):
        """Sets the allow_rebase_merge of this GithubRepository.


        :param allow_rebase_merge: The allow_rebase_merge of this GithubRepository.  # noqa: E501
        :type allow_rebase_merge: bool
        """

        self._allow_rebase_merge = allow_rebase_merge

    @property
    def allow_squash_merge(self):
        """Gets the allow_squash_merge of this GithubRepository.  # noqa: E501


        :return: The allow_squash_merge of this GithubRepository.  # noqa: E501
        :rtype: bool
        """
        return self._allow_squash_merge

    @allow_squash_merge.setter
    def allow_squash_merge(self, allow_squash_merge):
        """Sets the allow_squash_merge of this GithubRepository.


        :param allow_squash_merge: The allow_squash_merge of this GithubRepository.  # noqa: E501
        :type allow_squash_merge: bool
        """

        self._allow_squash_merge = allow_squash_merge

    @property
    def archive_url(self):
        """Gets the archive_url of this GithubRepository.  # noqa: E501


        :return: The archive_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._archive_url

    @archive_url.setter
    def archive_url(self, archive_url):
        """Sets the archive_url of this GithubRepository.


        :param archive_url: The archive_url of this GithubRepository.  # noqa: E501
        :type archive_url: str
        """

        self._archive_url = archive_url

    @property
    def archived(self):
        """Gets the archived of this GithubRepository.  # noqa: E501


        :return: The archived of this GithubRepository.  # noqa: E501
        :rtype: bool
        """
        return self._archived

    @archived.setter
    def archived(self, archived):
        """Sets the archived of this GithubRepository.


        :param archived: The archived of this GithubRepository.  # noqa: E501
        :type archived: bool
        """

        self._archived = archived

    @property
    def assignees_url(self):
        """Gets the assignees_url of this GithubRepository.  # noqa: E501


        :return: The assignees_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._assignees_url

    @assignees_url.setter
    def assignees_url(self, assignees_url):
        """Sets the assignees_url of this GithubRepository.


        :param assignees_url: The assignees_url of this GithubRepository.  # noqa: E501
        :type assignees_url: str
        """

        self._assignees_url = assignees_url

    @property
    def auto_init(self):
        """Gets the auto_init of this GithubRepository.  # noqa: E501


        :return: The auto_init of this GithubRepository.  # noqa: E501
        :rtype: bool
        """
        return self._auto_init

    @auto_init.setter
    def auto_init(self, auto_init):
        """Sets the auto_init of this GithubRepository.


        :param auto_init: The auto_init of this GithubRepository.  # noqa: E501
        :type auto_init: bool
        """

        self._auto_init = auto_init

    @property
    def blobs_url(self):
        """Gets the blobs_url of this GithubRepository.  # noqa: E501


        :return: The blobs_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._blobs_url

    @blobs_url.setter
    def blobs_url(self, blobs_url):
        """Sets the blobs_url of this GithubRepository.


        :param blobs_url: The blobs_url of this GithubRepository.  # noqa: E501
        :type blobs_url: str
        """

        self._blobs_url = blobs_url

    @property
    def branches_url(self):
        """Gets the branches_url of this GithubRepository.  # noqa: E501


        :return: The branches_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._branches_url

    @branches_url.setter
    def branches_url(self, branches_url):
        """Sets the branches_url of this GithubRepository.


        :param branches_url: The branches_url of this GithubRepository.  # noqa: E501
        :type branches_url: str
        """

        self._branches_url = branches_url

    @property
    def clone_url(self):
        """Gets the clone_url of this GithubRepository.  # noqa: E501


        :return: The clone_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._clone_url

    @clone_url.setter
    def clone_url(self, clone_url):
        """Sets the clone_url of this GithubRepository.


        :param clone_url: The clone_url of this GithubRepository.  # noqa: E501
        :type clone_url: str
        """

        self._clone_url = clone_url

    @property
    def code_of_conduct(self):
        """Gets the code_of_conduct of this GithubRepository.  # noqa: E501


        :return: The code_of_conduct of this GithubRepository.  # noqa: E501
        :rtype: GithubCodeOfConduct
        """
        return self._code_of_conduct

    @code_of_conduct.setter
    def code_of_conduct(self, code_of_conduct):
        """Sets the code_of_conduct of this GithubRepository.


        :param code_of_conduct: The code_of_conduct of this GithubRepository.  # noqa: E501
        :type code_of_conduct: GithubCodeOfConduct
        """

        self._code_of_conduct = code_of_conduct

    @property
    def collaborators_url(self):
        """Gets the collaborators_url of this GithubRepository.  # noqa: E501


        :return: The collaborators_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._collaborators_url

    @collaborators_url.setter
    def collaborators_url(self, collaborators_url):
        """Sets the collaborators_url of this GithubRepository.


        :param collaborators_url: The collaborators_url of this GithubRepository.  # noqa: E501
        :type collaborators_url: str
        """

        self._collaborators_url = collaborators_url

    @property
    def comments_url(self):
        """Gets the comments_url of this GithubRepository.  # noqa: E501


        :return: The comments_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._comments_url

    @comments_url.setter
    def comments_url(self, comments_url):
        """Sets the comments_url of this GithubRepository.


        :param comments_url: The comments_url of this GithubRepository.  # noqa: E501
        :type comments_url: str
        """

        self._comments_url = comments_url

    @property
    def commits_url(self):
        """Gets the commits_url of this GithubRepository.  # noqa: E501


        :return: The commits_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._commits_url

    @commits_url.setter
    def commits_url(self, commits_url):
        """Sets the commits_url of this GithubRepository.


        :param commits_url: The commits_url of this GithubRepository.  # noqa: E501
        :type commits_url: str
        """

        self._commits_url = commits_url

    @property
    def compare_url(self):
        """Gets the compare_url of this GithubRepository.  # noqa: E501


        :return: The compare_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._compare_url

    @compare_url.setter
    def compare_url(self, compare_url):
        """Sets the compare_url of this GithubRepository.


        :param compare_url: The compare_url of this GithubRepository.  # noqa: E501
        :type compare_url: str
        """

        self._compare_url = compare_url

    @property
    def contents_url(self):
        """Gets the contents_url of this GithubRepository.  # noqa: E501


        :return: The contents_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._contents_url

    @contents_url.setter
    def contents_url(self, contents_url):
        """Sets the contents_url of this GithubRepository.


        :param contents_url: The contents_url of this GithubRepository.  # noqa: E501
        :type contents_url: str
        """

        self._contents_url = contents_url

    @property
    def contributors_url(self):
        """Gets the contributors_url of this GithubRepository.  # noqa: E501


        :return: The contributors_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._contributors_url

    @contributors_url.setter
    def contributors_url(self, contributors_url):
        """Sets the contributors_url of this GithubRepository.


        :param contributors_url: The contributors_url of this GithubRepository.  # noqa: E501
        :type contributors_url: str
        """

        self._contributors_url = contributors_url

    @property
    def created_at(self):
        """Gets the created_at of this GithubRepository.  # noqa: E501


        :return: The created_at of this GithubRepository.  # noqa: E501
        :rtype: object
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this GithubRepository.


        :param created_at: The created_at of this GithubRepository.  # noqa: E501
        :type created_at: object
        """

        self._created_at = created_at

    @property
    def default_branch(self):
        """Gets the default_branch of this GithubRepository.  # noqa: E501


        :return: The default_branch of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._default_branch

    @default_branch.setter
    def default_branch(self, default_branch):
        """Sets the default_branch of this GithubRepository.


        :param default_branch: The default_branch of this GithubRepository.  # noqa: E501
        :type default_branch: str
        """

        self._default_branch = default_branch

    @property
    def delete_branch_on_merge(self):
        """Gets the delete_branch_on_merge of this GithubRepository.  # noqa: E501


        :return: The delete_branch_on_merge of this GithubRepository.  # noqa: E501
        :rtype: bool
        """
        return self._delete_branch_on_merge

    @delete_branch_on_merge.setter
    def delete_branch_on_merge(self, delete_branch_on_merge):
        """Sets the delete_branch_on_merge of this GithubRepository.


        :param delete_branch_on_merge: The delete_branch_on_merge of this GithubRepository.  # noqa: E501
        :type delete_branch_on_merge: bool
        """

        self._delete_branch_on_merge = delete_branch_on_merge

    @property
    def deployments_url(self):
        """Gets the deployments_url of this GithubRepository.  # noqa: E501


        :return: The deployments_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._deployments_url

    @deployments_url.setter
    def deployments_url(self, deployments_url):
        """Sets the deployments_url of this GithubRepository.


        :param deployments_url: The deployments_url of this GithubRepository.  # noqa: E501
        :type deployments_url: str
        """

        self._deployments_url = deployments_url

    @property
    def description(self):
        """Gets the description of this GithubRepository.  # noqa: E501


        :return: The description of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this GithubRepository.


        :param description: The description of this GithubRepository.  # noqa: E501
        :type description: str
        """

        self._description = description

    @property
    def disabled(self):
        """Gets the disabled of this GithubRepository.  # noqa: E501


        :return: The disabled of this GithubRepository.  # noqa: E501
        :rtype: bool
        """
        return self._disabled

    @disabled.setter
    def disabled(self, disabled):
        """Sets the disabled of this GithubRepository.


        :param disabled: The disabled of this GithubRepository.  # noqa: E501
        :type disabled: bool
        """

        self._disabled = disabled

    @property
    def downloads_url(self):
        """Gets the downloads_url of this GithubRepository.  # noqa: E501


        :return: The downloads_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._downloads_url

    @downloads_url.setter
    def downloads_url(self, downloads_url):
        """Sets the downloads_url of this GithubRepository.


        :param downloads_url: The downloads_url of this GithubRepository.  # noqa: E501
        :type downloads_url: str
        """

        self._downloads_url = downloads_url

    @property
    def events_url(self):
        """Gets the events_url of this GithubRepository.  # noqa: E501


        :return: The events_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._events_url

    @events_url.setter
    def events_url(self, events_url):
        """Sets the events_url of this GithubRepository.


        :param events_url: The events_url of this GithubRepository.  # noqa: E501
        :type events_url: str
        """

        self._events_url = events_url

    @property
    def fork(self):
        """Gets the fork of this GithubRepository.  # noqa: E501


        :return: The fork of this GithubRepository.  # noqa: E501
        :rtype: bool
        """
        return self._fork

    @fork.setter
    def fork(self, fork):
        """Sets the fork of this GithubRepository.


        :param fork: The fork of this GithubRepository.  # noqa: E501
        :type fork: bool
        """

        self._fork = fork

    @property
    def forks_count(self):
        """Gets the forks_count of this GithubRepository.  # noqa: E501


        :return: The forks_count of this GithubRepository.  # noqa: E501
        :rtype: int
        """
        return self._forks_count

    @forks_count.setter
    def forks_count(self, forks_count):
        """Sets the forks_count of this GithubRepository.


        :param forks_count: The forks_count of this GithubRepository.  # noqa: E501
        :type forks_count: int
        """

        self._forks_count = forks_count

    @property
    def forks_url(self):
        """Gets the forks_url of this GithubRepository.  # noqa: E501


        :return: The forks_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._forks_url

    @forks_url.setter
    def forks_url(self, forks_url):
        """Sets the forks_url of this GithubRepository.


        :param forks_url: The forks_url of this GithubRepository.  # noqa: E501
        :type forks_url: str
        """

        self._forks_url = forks_url

    @property
    def full_name(self):
        """Gets the full_name of this GithubRepository.  # noqa: E501


        :return: The full_name of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._full_name

    @full_name.setter
    def full_name(self, full_name):
        """Sets the full_name of this GithubRepository.


        :param full_name: The full_name of this GithubRepository.  # noqa: E501
        :type full_name: str
        """

        self._full_name = full_name

    @property
    def git_commits_url(self):
        """Gets the git_commits_url of this GithubRepository.  # noqa: E501


        :return: The git_commits_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._git_commits_url

    @git_commits_url.setter
    def git_commits_url(self, git_commits_url):
        """Sets the git_commits_url of this GithubRepository.


        :param git_commits_url: The git_commits_url of this GithubRepository.  # noqa: E501
        :type git_commits_url: str
        """

        self._git_commits_url = git_commits_url

    @property
    def git_refs_url(self):
        """Gets the git_refs_url of this GithubRepository.  # noqa: E501


        :return: The git_refs_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._git_refs_url

    @git_refs_url.setter
    def git_refs_url(self, git_refs_url):
        """Sets the git_refs_url of this GithubRepository.


        :param git_refs_url: The git_refs_url of this GithubRepository.  # noqa: E501
        :type git_refs_url: str
        """

        self._git_refs_url = git_refs_url

    @property
    def git_tags_url(self):
        """Gets the git_tags_url of this GithubRepository.  # noqa: E501


        :return: The git_tags_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._git_tags_url

    @git_tags_url.setter
    def git_tags_url(self, git_tags_url):
        """Sets the git_tags_url of this GithubRepository.


        :param git_tags_url: The git_tags_url of this GithubRepository.  # noqa: E501
        :type git_tags_url: str
        """

        self._git_tags_url = git_tags_url

    @property
    def git_url(self):
        """Gets the git_url of this GithubRepository.  # noqa: E501


        :return: The git_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._git_url

    @git_url.setter
    def git_url(self, git_url):
        """Sets the git_url of this GithubRepository.


        :param git_url: The git_url of this GithubRepository.  # noqa: E501
        :type git_url: str
        """

        self._git_url = git_url

    @property
    def gitignore_template(self):
        """Gets the gitignore_template of this GithubRepository.  # noqa: E501


        :return: The gitignore_template of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._gitignore_template

    @gitignore_template.setter
    def gitignore_template(self, gitignore_template):
        """Sets the gitignore_template of this GithubRepository.


        :param gitignore_template: The gitignore_template of this GithubRepository.  # noqa: E501
        :type gitignore_template: str
        """

        self._gitignore_template = gitignore_template

    @property
    def has_downloads(self):
        """Gets the has_downloads of this GithubRepository.  # noqa: E501


        :return: The has_downloads of this GithubRepository.  # noqa: E501
        :rtype: bool
        """
        return self._has_downloads

    @has_downloads.setter
    def has_downloads(self, has_downloads):
        """Sets the has_downloads of this GithubRepository.


        :param has_downloads: The has_downloads of this GithubRepository.  # noqa: E501
        :type has_downloads: bool
        """

        self._has_downloads = has_downloads

    @property
    def has_issues(self):
        """Gets the has_issues of this GithubRepository.  # noqa: E501


        :return: The has_issues of this GithubRepository.  # noqa: E501
        :rtype: bool
        """
        return self._has_issues

    @has_issues.setter
    def has_issues(self, has_issues):
        """Sets the has_issues of this GithubRepository.


        :param has_issues: The has_issues of this GithubRepository.  # noqa: E501
        :type has_issues: bool
        """

        self._has_issues = has_issues

    @property
    def has_pages(self):
        """Gets the has_pages of this GithubRepository.  # noqa: E501


        :return: The has_pages of this GithubRepository.  # noqa: E501
        :rtype: bool
        """
        return self._has_pages

    @has_pages.setter
    def has_pages(self, has_pages):
        """Sets the has_pages of this GithubRepository.


        :param has_pages: The has_pages of this GithubRepository.  # noqa: E501
        :type has_pages: bool
        """

        self._has_pages = has_pages

    @property
    def has_projects(self):
        """Gets the has_projects of this GithubRepository.  # noqa: E501


        :return: The has_projects of this GithubRepository.  # noqa: E501
        :rtype: bool
        """
        return self._has_projects

    @has_projects.setter
    def has_projects(self, has_projects):
        """Sets the has_projects of this GithubRepository.


        :param has_projects: The has_projects of this GithubRepository.  # noqa: E501
        :type has_projects: bool
        """

        self._has_projects = has_projects

    @property
    def has_wiki(self):
        """Gets the has_wiki of this GithubRepository.  # noqa: E501


        :return: The has_wiki of this GithubRepository.  # noqa: E501
        :rtype: bool
        """
        return self._has_wiki

    @has_wiki.setter
    def has_wiki(self, has_wiki):
        """Sets the has_wiki of this GithubRepository.


        :param has_wiki: The has_wiki of this GithubRepository.  # noqa: E501
        :type has_wiki: bool
        """

        self._has_wiki = has_wiki

    @property
    def homepage(self):
        """Gets the homepage of this GithubRepository.  # noqa: E501


        :return: The homepage of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._homepage

    @homepage.setter
    def homepage(self, homepage):
        """Sets the homepage of this GithubRepository.


        :param homepage: The homepage of this GithubRepository.  # noqa: E501
        :type homepage: str
        """

        self._homepage = homepage

    @property
    def hooks_url(self):
        """Gets the hooks_url of this GithubRepository.  # noqa: E501


        :return: The hooks_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._hooks_url

    @hooks_url.setter
    def hooks_url(self, hooks_url):
        """Sets the hooks_url of this GithubRepository.


        :param hooks_url: The hooks_url of this GithubRepository.  # noqa: E501
        :type hooks_url: str
        """

        self._hooks_url = hooks_url

    @property
    def html_url(self):
        """Gets the html_url of this GithubRepository.  # noqa: E501


        :return: The html_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._html_url

    @html_url.setter
    def html_url(self, html_url):
        """Sets the html_url of this GithubRepository.


        :param html_url: The html_url of this GithubRepository.  # noqa: E501
        :type html_url: str
        """

        self._html_url = html_url

    @property
    def id(self):
        """Gets the id of this GithubRepository.  # noqa: E501


        :return: The id of this GithubRepository.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this GithubRepository.


        :param id: The id of this GithubRepository.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def is_template(self):
        """Gets the is_template of this GithubRepository.  # noqa: E501


        :return: The is_template of this GithubRepository.  # noqa: E501
        :rtype: bool
        """
        return self._is_template

    @is_template.setter
    def is_template(self, is_template):
        """Sets the is_template of this GithubRepository.


        :param is_template: The is_template of this GithubRepository.  # noqa: E501
        :type is_template: bool
        """

        self._is_template = is_template

    @property
    def issue_comment_url(self):
        """Gets the issue_comment_url of this GithubRepository.  # noqa: E501


        :return: The issue_comment_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._issue_comment_url

    @issue_comment_url.setter
    def issue_comment_url(self, issue_comment_url):
        """Sets the issue_comment_url of this GithubRepository.


        :param issue_comment_url: The issue_comment_url of this GithubRepository.  # noqa: E501
        :type issue_comment_url: str
        """

        self._issue_comment_url = issue_comment_url

    @property
    def issue_events_url(self):
        """Gets the issue_events_url of this GithubRepository.  # noqa: E501


        :return: The issue_events_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._issue_events_url

    @issue_events_url.setter
    def issue_events_url(self, issue_events_url):
        """Sets the issue_events_url of this GithubRepository.


        :param issue_events_url: The issue_events_url of this GithubRepository.  # noqa: E501
        :type issue_events_url: str
        """

        self._issue_events_url = issue_events_url

    @property
    def issues_url(self):
        """Gets the issues_url of this GithubRepository.  # noqa: E501


        :return: The issues_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._issues_url

    @issues_url.setter
    def issues_url(self, issues_url):
        """Sets the issues_url of this GithubRepository.


        :param issues_url: The issues_url of this GithubRepository.  # noqa: E501
        :type issues_url: str
        """

        self._issues_url = issues_url

    @property
    def keys_url(self):
        """Gets the keys_url of this GithubRepository.  # noqa: E501


        :return: The keys_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._keys_url

    @keys_url.setter
    def keys_url(self, keys_url):
        """Sets the keys_url of this GithubRepository.


        :param keys_url: The keys_url of this GithubRepository.  # noqa: E501
        :type keys_url: str
        """

        self._keys_url = keys_url

    @property
    def labels_url(self):
        """Gets the labels_url of this GithubRepository.  # noqa: E501


        :return: The labels_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._labels_url

    @labels_url.setter
    def labels_url(self, labels_url):
        """Sets the labels_url of this GithubRepository.


        :param labels_url: The labels_url of this GithubRepository.  # noqa: E501
        :type labels_url: str
        """

        self._labels_url = labels_url

    @property
    def language(self):
        """Gets the language of this GithubRepository.  # noqa: E501


        :return: The language of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._language

    @language.setter
    def language(self, language):
        """Sets the language of this GithubRepository.


        :param language: The language of this GithubRepository.  # noqa: E501
        :type language: str
        """

        self._language = language

    @property
    def languages_url(self):
        """Gets the languages_url of this GithubRepository.  # noqa: E501


        :return: The languages_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._languages_url

    @languages_url.setter
    def languages_url(self, languages_url):
        """Sets the languages_url of this GithubRepository.


        :param languages_url: The languages_url of this GithubRepository.  # noqa: E501
        :type languages_url: str
        """

        self._languages_url = languages_url

    @property
    def license(self):
        """Gets the license of this GithubRepository.  # noqa: E501


        :return: The license of this GithubRepository.  # noqa: E501
        :rtype: GithubLicense
        """
        return self._license

    @license.setter
    def license(self, license):
        """Sets the license of this GithubRepository.


        :param license: The license of this GithubRepository.  # noqa: E501
        :type license: GithubLicense
        """

        self._license = license

    @property
    def license_template(self):
        """Gets the license_template of this GithubRepository.  # noqa: E501


        :return: The license_template of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._license_template

    @license_template.setter
    def license_template(self, license_template):
        """Sets the license_template of this GithubRepository.


        :param license_template: The license_template of this GithubRepository.  # noqa: E501
        :type license_template: str
        """

        self._license_template = license_template

    @property
    def master_branch(self):
        """Gets the master_branch of this GithubRepository.  # noqa: E501


        :return: The master_branch of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._master_branch

    @master_branch.setter
    def master_branch(self, master_branch):
        """Sets the master_branch of this GithubRepository.


        :param master_branch: The master_branch of this GithubRepository.  # noqa: E501
        :type master_branch: str
        """

        self._master_branch = master_branch

    @property
    def merges_url(self):
        """Gets the merges_url of this GithubRepository.  # noqa: E501


        :return: The merges_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._merges_url

    @merges_url.setter
    def merges_url(self, merges_url):
        """Sets the merges_url of this GithubRepository.


        :param merges_url: The merges_url of this GithubRepository.  # noqa: E501
        :type merges_url: str
        """

        self._merges_url = merges_url

    @property
    def milestones_url(self):
        """Gets the milestones_url of this GithubRepository.  # noqa: E501


        :return: The milestones_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._milestones_url

    @milestones_url.setter
    def milestones_url(self, milestones_url):
        """Sets the milestones_url of this GithubRepository.


        :param milestones_url: The milestones_url of this GithubRepository.  # noqa: E501
        :type milestones_url: str
        """

        self._milestones_url = milestones_url

    @property
    def mirror_url(self):
        """Gets the mirror_url of this GithubRepository.  # noqa: E501


        :return: The mirror_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._mirror_url

    @mirror_url.setter
    def mirror_url(self, mirror_url):
        """Sets the mirror_url of this GithubRepository.


        :param mirror_url: The mirror_url of this GithubRepository.  # noqa: E501
        :type mirror_url: str
        """

        self._mirror_url = mirror_url

    @property
    def name(self):
        """Gets the name of this GithubRepository.  # noqa: E501


        :return: The name of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this GithubRepository.


        :param name: The name of this GithubRepository.  # noqa: E501
        :type name: str
        """

        self._name = name

    @property
    def network_count(self):
        """Gets the network_count of this GithubRepository.  # noqa: E501


        :return: The network_count of this GithubRepository.  # noqa: E501
        :rtype: int
        """
        return self._network_count

    @network_count.setter
    def network_count(self, network_count):
        """Sets the network_count of this GithubRepository.


        :param network_count: The network_count of this GithubRepository.  # noqa: E501
        :type network_count: int
        """

        self._network_count = network_count

    @property
    def node_id(self):
        """Gets the node_id of this GithubRepository.  # noqa: E501


        :return: The node_id of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._node_id

    @node_id.setter
    def node_id(self, node_id):
        """Sets the node_id of this GithubRepository.


        :param node_id: The node_id of this GithubRepository.  # noqa: E501
        :type node_id: str
        """

        self._node_id = node_id

    @property
    def notifications_url(self):
        """Gets the notifications_url of this GithubRepository.  # noqa: E501


        :return: The notifications_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._notifications_url

    @notifications_url.setter
    def notifications_url(self, notifications_url):
        """Sets the notifications_url of this GithubRepository.


        :param notifications_url: The notifications_url of this GithubRepository.  # noqa: E501
        :type notifications_url: str
        """

        self._notifications_url = notifications_url

    @property
    def open_issues(self):
        """Gets the open_issues of this GithubRepository.  # noqa: E501


        :return: The open_issues of this GithubRepository.  # noqa: E501
        :rtype: int
        """
        return self._open_issues

    @open_issues.setter
    def open_issues(self, open_issues):
        """Sets the open_issues of this GithubRepository.


        :param open_issues: The open_issues of this GithubRepository.  # noqa: E501
        :type open_issues: int
        """

        self._open_issues = open_issues

    @property
    def open_issues_count(self):
        """Gets the open_issues_count of this GithubRepository.  # noqa: E501


        :return: The open_issues_count of this GithubRepository.  # noqa: E501
        :rtype: int
        """
        return self._open_issues_count

    @open_issues_count.setter
    def open_issues_count(self, open_issues_count):
        """Sets the open_issues_count of this GithubRepository.


        :param open_issues_count: The open_issues_count of this GithubRepository.  # noqa: E501
        :type open_issues_count: int
        """

        self._open_issues_count = open_issues_count

    @property
    def organization(self):
        """Gets the organization of this GithubRepository.  # noqa: E501


        :return: The organization of this GithubRepository.  # noqa: E501
        :rtype: GithubOrganization
        """
        return self._organization

    @organization.setter
    def organization(self, organization):
        """Sets the organization of this GithubRepository.


        :param organization: The organization of this GithubRepository.  # noqa: E501
        :type organization: GithubOrganization
        """

        self._organization = organization

    @property
    def owner(self):
        """Gets the owner of this GithubRepository.  # noqa: E501


        :return: The owner of this GithubRepository.  # noqa: E501
        :rtype: GithubUser
        """
        return self._owner

    @owner.setter
    def owner(self, owner):
        """Sets the owner of this GithubRepository.


        :param owner: The owner of this GithubRepository.  # noqa: E501
        :type owner: GithubUser
        """

        self._owner = owner

    @property
    def parent(self):
        """Gets the parent of this GithubRepository.  # noqa: E501


        :return: The parent of this GithubRepository.  # noqa: E501
        :rtype: GithubRepository
        """
        return self._parent

    @parent.setter
    def parent(self, parent):
        """Sets the parent of this GithubRepository.


        :param parent: The parent of this GithubRepository.  # noqa: E501
        :type parent: GithubRepository
        """

        self._parent = parent

    @property
    def permissions(self):
        """Gets the permissions of this GithubRepository.  # noqa: E501


        :return: The permissions of this GithubRepository.  # noqa: E501
        :rtype: dict(str, bool)
        """
        return self._permissions

    @permissions.setter
    def permissions(self, permissions):
        """Sets the permissions of this GithubRepository.


        :param permissions: The permissions of this GithubRepository.  # noqa: E501
        :type permissions: dict(str, bool)
        """

        self._permissions = permissions

    @property
    def private(self):
        """Gets the private of this GithubRepository.  # noqa: E501


        :return: The private of this GithubRepository.  # noqa: E501
        :rtype: bool
        """
        return self._private

    @private.setter
    def private(self, private):
        """Sets the private of this GithubRepository.


        :param private: The private of this GithubRepository.  # noqa: E501
        :type private: bool
        """

        self._private = private

    @property
    def pulls_url(self):
        """Gets the pulls_url of this GithubRepository.  # noqa: E501


        :return: The pulls_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._pulls_url

    @pulls_url.setter
    def pulls_url(self, pulls_url):
        """Sets the pulls_url of this GithubRepository.


        :param pulls_url: The pulls_url of this GithubRepository.  # noqa: E501
        :type pulls_url: str
        """

        self._pulls_url = pulls_url

    @property
    def pushed_at(self):
        """Gets the pushed_at of this GithubRepository.  # noqa: E501


        :return: The pushed_at of this GithubRepository.  # noqa: E501
        :rtype: object
        """
        return self._pushed_at

    @pushed_at.setter
    def pushed_at(self, pushed_at):
        """Sets the pushed_at of this GithubRepository.


        :param pushed_at: The pushed_at of this GithubRepository.  # noqa: E501
        :type pushed_at: object
        """

        self._pushed_at = pushed_at

    @property
    def releases_url(self):
        """Gets the releases_url of this GithubRepository.  # noqa: E501


        :return: The releases_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._releases_url

    @releases_url.setter
    def releases_url(self, releases_url):
        """Sets the releases_url of this GithubRepository.


        :param releases_url: The releases_url of this GithubRepository.  # noqa: E501
        :type releases_url: str
        """

        self._releases_url = releases_url

    @property
    def size(self):
        """Gets the size of this GithubRepository.  # noqa: E501


        :return: The size of this GithubRepository.  # noqa: E501
        :rtype: int
        """
        return self._size

    @size.setter
    def size(self, size):
        """Sets the size of this GithubRepository.


        :param size: The size of this GithubRepository.  # noqa: E501
        :type size: int
        """

        self._size = size

    @property
    def source(self):
        """Gets the source of this GithubRepository.  # noqa: E501


        :return: The source of this GithubRepository.  # noqa: E501
        :rtype: GithubRepository
        """
        return self._source

    @source.setter
    def source(self, source):
        """Sets the source of this GithubRepository.


        :param source: The source of this GithubRepository.  # noqa: E501
        :type source: GithubRepository
        """

        self._source = source

    @property
    def ssh_url(self):
        """Gets the ssh_url of this GithubRepository.  # noqa: E501


        :return: The ssh_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._ssh_url

    @ssh_url.setter
    def ssh_url(self, ssh_url):
        """Sets the ssh_url of this GithubRepository.


        :param ssh_url: The ssh_url of this GithubRepository.  # noqa: E501
        :type ssh_url: str
        """

        self._ssh_url = ssh_url

    @property
    def stargazers_count(self):
        """Gets the stargazers_count of this GithubRepository.  # noqa: E501


        :return: The stargazers_count of this GithubRepository.  # noqa: E501
        :rtype: int
        """
        return self._stargazers_count

    @stargazers_count.setter
    def stargazers_count(self, stargazers_count):
        """Sets the stargazers_count of this GithubRepository.


        :param stargazers_count: The stargazers_count of this GithubRepository.  # noqa: E501
        :type stargazers_count: int
        """

        self._stargazers_count = stargazers_count

    @property
    def stargazers_url(self):
        """Gets the stargazers_url of this GithubRepository.  # noqa: E501


        :return: The stargazers_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._stargazers_url

    @stargazers_url.setter
    def stargazers_url(self, stargazers_url):
        """Sets the stargazers_url of this GithubRepository.


        :param stargazers_url: The stargazers_url of this GithubRepository.  # noqa: E501
        :type stargazers_url: str
        """

        self._stargazers_url = stargazers_url

    @property
    def statuses_url(self):
        """Gets the statuses_url of this GithubRepository.  # noqa: E501


        :return: The statuses_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._statuses_url

    @statuses_url.setter
    def statuses_url(self, statuses_url):
        """Sets the statuses_url of this GithubRepository.


        :param statuses_url: The statuses_url of this GithubRepository.  # noqa: E501
        :type statuses_url: str
        """

        self._statuses_url = statuses_url

    @property
    def subscribers_count(self):
        """Gets the subscribers_count of this GithubRepository.  # noqa: E501


        :return: The subscribers_count of this GithubRepository.  # noqa: E501
        :rtype: int
        """
        return self._subscribers_count

    @subscribers_count.setter
    def subscribers_count(self, subscribers_count):
        """Sets the subscribers_count of this GithubRepository.


        :param subscribers_count: The subscribers_count of this GithubRepository.  # noqa: E501
        :type subscribers_count: int
        """

        self._subscribers_count = subscribers_count

    @property
    def subscribers_url(self):
        """Gets the subscribers_url of this GithubRepository.  # noqa: E501


        :return: The subscribers_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._subscribers_url

    @subscribers_url.setter
    def subscribers_url(self, subscribers_url):
        """Sets the subscribers_url of this GithubRepository.


        :param subscribers_url: The subscribers_url of this GithubRepository.  # noqa: E501
        :type subscribers_url: str
        """

        self._subscribers_url = subscribers_url

    @property
    def subscription_url(self):
        """Gets the subscription_url of this GithubRepository.  # noqa: E501


        :return: The subscription_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._subscription_url

    @subscription_url.setter
    def subscription_url(self, subscription_url):
        """Sets the subscription_url of this GithubRepository.


        :param subscription_url: The subscription_url of this GithubRepository.  # noqa: E501
        :type subscription_url: str
        """

        self._subscription_url = subscription_url

    @property
    def svn_url(self):
        """Gets the svn_url of this GithubRepository.  # noqa: E501


        :return: The svn_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._svn_url

    @svn_url.setter
    def svn_url(self, svn_url):
        """Sets the svn_url of this GithubRepository.


        :param svn_url: The svn_url of this GithubRepository.  # noqa: E501
        :type svn_url: str
        """

        self._svn_url = svn_url

    @property
    def tags_url(self):
        """Gets the tags_url of this GithubRepository.  # noqa: E501


        :return: The tags_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._tags_url

    @tags_url.setter
    def tags_url(self, tags_url):
        """Sets the tags_url of this GithubRepository.


        :param tags_url: The tags_url of this GithubRepository.  # noqa: E501
        :type tags_url: str
        """

        self._tags_url = tags_url

    @property
    def team_id(self):
        """Gets the team_id of this GithubRepository.  # noqa: E501


        :return: The team_id of this GithubRepository.  # noqa: E501
        :rtype: int
        """
        return self._team_id

    @team_id.setter
    def team_id(self, team_id):
        """Sets the team_id of this GithubRepository.


        :param team_id: The team_id of this GithubRepository.  # noqa: E501
        :type team_id: int
        """

        self._team_id = team_id

    @property
    def teams_url(self):
        """Gets the teams_url of this GithubRepository.  # noqa: E501


        :return: The teams_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._teams_url

    @teams_url.setter
    def teams_url(self, teams_url):
        """Sets the teams_url of this GithubRepository.


        :param teams_url: The teams_url of this GithubRepository.  # noqa: E501
        :type teams_url: str
        """

        self._teams_url = teams_url

    @property
    def template_repository(self):
        """Gets the template_repository of this GithubRepository.  # noqa: E501


        :return: The template_repository of this GithubRepository.  # noqa: E501
        :rtype: GithubRepository
        """
        return self._template_repository

    @template_repository.setter
    def template_repository(self, template_repository):
        """Sets the template_repository of this GithubRepository.


        :param template_repository: The template_repository of this GithubRepository.  # noqa: E501
        :type template_repository: GithubRepository
        """

        self._template_repository = template_repository

    @property
    def text_matches(self):
        """Gets the text_matches of this GithubRepository.  # noqa: E501


        :return: The text_matches of this GithubRepository.  # noqa: E501
        :rtype: list[GithubTextMatch]
        """
        return self._text_matches

    @text_matches.setter
    def text_matches(self, text_matches):
        """Sets the text_matches of this GithubRepository.


        :param text_matches: The text_matches of this GithubRepository.  # noqa: E501
        :type text_matches: list[GithubTextMatch]
        """

        self._text_matches = text_matches

    @property
    def topics(self):
        """Gets the topics of this GithubRepository.  # noqa: E501


        :return: The topics of this GithubRepository.  # noqa: E501
        :rtype: list[str]
        """
        return self._topics

    @topics.setter
    def topics(self, topics):
        """Sets the topics of this GithubRepository.


        :param topics: The topics of this GithubRepository.  # noqa: E501
        :type topics: list[str]
        """

        self._topics = topics

    @property
    def trees_url(self):
        """Gets the trees_url of this GithubRepository.  # noqa: E501


        :return: The trees_url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._trees_url

    @trees_url.setter
    def trees_url(self, trees_url):
        """Sets the trees_url of this GithubRepository.


        :param trees_url: The trees_url of this GithubRepository.  # noqa: E501
        :type trees_url: str
        """

        self._trees_url = trees_url

    @property
    def updated_at(self):
        """Gets the updated_at of this GithubRepository.  # noqa: E501


        :return: The updated_at of this GithubRepository.  # noqa: E501
        :rtype: object
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this GithubRepository.


        :param updated_at: The updated_at of this GithubRepository.  # noqa: E501
        :type updated_at: object
        """

        self._updated_at = updated_at

    @property
    def url(self):
        """Gets the url of this GithubRepository.  # noqa: E501


        :return: The url of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this GithubRepository.


        :param url: The url of this GithubRepository.  # noqa: E501
        :type url: str
        """

        self._url = url

    @property
    def visibility(self):
        """Gets the visibility of this GithubRepository.  # noqa: E501


        :return: The visibility of this GithubRepository.  # noqa: E501
        :rtype: str
        """
        return self._visibility

    @visibility.setter
    def visibility(self, visibility):
        """Sets the visibility of this GithubRepository.


        :param visibility: The visibility of this GithubRepository.  # noqa: E501
        :type visibility: str
        """

        self._visibility = visibility

    @property
    def watchers(self):
        """Gets the watchers of this GithubRepository.  # noqa: E501


        :return: The watchers of this GithubRepository.  # noqa: E501
        :rtype: int
        """
        return self._watchers

    @watchers.setter
    def watchers(self, watchers):
        """Sets the watchers of this GithubRepository.


        :param watchers: The watchers of this GithubRepository.  # noqa: E501
        :type watchers: int
        """

        self._watchers = watchers

    @property
    def watchers_count(self):
        """Gets the watchers_count of this GithubRepository.  # noqa: E501


        :return: The watchers_count of this GithubRepository.  # noqa: E501
        :rtype: int
        """
        return self._watchers_count

    @watchers_count.setter
    def watchers_count(self, watchers_count):
        """Sets the watchers_count of this GithubRepository.


        :param watchers_count: The watchers_count of this GithubRepository.  # noqa: E501
        :type watchers_count: int
        """

        self._watchers_count = watchers_count

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GithubRepository):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, GithubRepository):
            return True

        return self.to_dict() != other.to_dict()



import json

from openapi_client.exceptions import ApiException

DEFAULT_ERROR_MESSAGE = "An unexpected exception occurred. Use VESSL_LOG=DEBUG to view stack trace. (`pip install --upgrade vessl` might resolve this issue.)"


class SavvihubException(Exception):
    def __init__(self, message=DEFAULT_ERROR_MESSAGE, exit_code=1):
        self.message = message
        self.exit_code = exit_code
        super().__init__(message)


class SavvihubApiException(SavvihubException):
    @classmethod
    def convert_api_exception(
        cls, api_exception: ApiException
    ) -> "SavvihubApiException":
        try:
            body = json.loads(api_exception.body)
        except json.JSONDecodeError:
            return cls()

        body_code = body.get("code")
        body_message = body.get("message", "")
        message = (
            f"{body_code} ({api_exception.status})"
            f"{': ' + body_message if body_message else ''}"
        )

        fields = body.get("fields")
        if fields:
            additional_messages = []
            for field in fields:
                field_name = field.get("name", "")
                field_value = field.get("value", "")
                field_message = field.get("message")
                additional_messages.append(
                    f"{field_name}: {field_value}"
                    f"{'(' + field_message + ')' if field_message else ''}"
                )
            message += f" {', '.join(additional_messages)}."

        return cls(message=message)


class GitError(SavvihubException):
    pass


class TimeoutError(SavvihubException):
    pass


class InvalidDatasetError(SavvihubException):
    pass


class InvalidKernelImageError(SavvihubException):
    pass


class InvalidKernelResourceSpecError(SavvihubException):
    pass


class InvalidOrganizationError(SavvihubException):
    pass


class InvalidExperimentError(SavvihubException):
    pass


class InvalidProjectError(SavvihubException):
    pass


class InvalidTokenError(SavvihubException):
    pass


class InvalidVolumeFileError(SavvihubException):
    pass


class InvalidVolumeMountError(SavvihubException):
    pass


class InvalidWorkspaceError(SavvihubException):
    pass


class InvalidParamsError(SavvihubException):
    pass
